###############################################################################
# Copyright (c) 2018, Urban Magnetometry Group.
#
# Produced at the University of California at Berkeley.
#
# Written by V. Dumont (vincentdumont11@gmail.com).
#
# All rights reserved.
#
# This file is part of NURI.
#
# For details, see github.com/vincentdumont/nuri.
#
# For details about use and distribution, please read NURI/LICENSE.
###############################################################################
import sys,os,numpy
import matplotlib.pyplot as plt
import matplotlib.dates as md
from datetime import datetime,timedelta

def get_activity(year,month):
    """
    This operation will display the active periods for which data are
    available from every sensors.
    
    Parameters
    ----------
    year : int
      Year to display activity from.
    month : int
      Month to display activity from.

    Examples
    --------
      >>> nuri active -y 2017 -m 10
    """
    # List all the months
    dates = numpy.empty((0,5))
    y0    = int(year)
    m0    = int(month)
    d0    = datetime(y0,m0,1)
    y1    = y0   if m0<12 else y0+1
    m1    = m0+1 if m0<12 else 1
    d1    = datetime(y1,m1,1)-timedelta(hours=1)
    dt    = timedelta(hours=1)
    dates = numpy.arange(d0,d1,dt)
    # Download metadata from Google Drive
    sys.stderr.write('Retrieve information from Google Drive...')
    os.system('skicka ls -r /MagneticFieldData/%s/%s/ > data'%(y0,m0))
    data = numpy.loadtxt('data',dtype=str,delimiter='\n')
    print(sys.stderr,' done!')
    # List file path for each date and each station
    sys.stderr.write('Select active hours for each station...')
    st0,st1,st2,st3,st4 = [],[],[],[],[]
    for d in dates:
        year  = d.astype(object).year
        month = d.astype(object).month
        day   = d.astype(object).day
        hour  = d.astype(object).hour
        path  = 'MagneticFieldData/%i/%i/%i/%i/'%(year,month,day,hour)
        fname = '%i-%i-%i_%i-xx.zip'%(year,month,day,hour)
        st0.append(path+'NURI-station/'   +fname)
        st1.append(path+'NURI-station-01/'+fname)
        st2.append(path+'NURI-station-02/'+fname)
        st3.append(path+'NURI-station-03/'+fname)
        st4.append(path+'NURI-station-04/'+fname)
    st0 = numpy.array([1 if path in data else 0 for path in st0])
    st1 = numpy.array([1 if path in data else 0 for path in st1])
    st2 = numpy.array([1 if path in data else 0 for path in st2])
    st3 = numpy.array([1 if path in data else 0 for path in st3])
    st4 = numpy.array([1 if path in data else 0 for path in st4])
    print(sys.stderr,' done!')
    # Write down information in text file
    print('Save information in ASCII file...')
    o = open('%i-%02i.txt'%(y0,m0),'w')
    for d in dates:
        year  = d.astype(object).year
        month = d.astype(object).month
        day   = d.astype(object).day
        hour  = d.astype(object).hour
        path  = 'MagneticFieldData/%i/%i/%i/%i/'%(year,month,day,hour)
        fname = '%i-%i-%i_%i-xx.zip'%(year,month,day,hour)
        o.write('%i-%02i-%02i_%02i'%(year,month,day,hour))
        for i in range(5):
            station = 'NURI-station-%02i'%i if i>0 else 'NURI-station'
            text = station if path+station+'/'+fname in data else '-'
            o.write('  {:<15}'.format(text))
        o.write('\n')
    o.close()
    dates = [d.astype(object) for d in dates]
    plt.rc('font', size=2, family='serif')
    plt.rc('axes', labelsize=10, linewidth=0.2)
    plt.rc('legend', fontsize=2, handlelength=10)
    plt.rc('xtick', labelsize=7)
    plt.rc('ytick', labelsize=7)
    plt.rc('lines', lw=0.2, mew=0.2)
    plt.rc('grid', linewidth=0.2)
    fig = plt.figure(figsize=(10,6))
    plt.subplots_adjust(left=0.07, right=0.95, bottom=0.1, top=0.96, hspace=0.2, wspace=0)
    print('Plot active time for station 1...')
    ax1 = fig.add_subplot(511)
    ax1.bar(dates,st1,width=1/24.,edgecolor='none')
    ax1.tick_params(direction='in')
    ax1.set_ylabel('Station 1')
    ax1.xaxis_date()
    plt.yticks([])
    ax1.grid()
    print('Plot active time for station 2...')
    ax = fig.add_subplot(512,sharex=ax1,sharey=ax1)
    ax.bar(dates,st2,width=1/24.,edgecolor='none')
    ax.tick_params(direction='in')
    ax.set_ylabel('Station 2')
    ax.xaxis_date()
    plt.yticks([])
    ax.grid()
    print('Plot active time for station 3...')
    ax = fig.add_subplot(513,sharex=ax1,sharey=ax1)
    ax.bar(dates,st3,width=1/24.,edgecolor='none')
    ax.tick_params(direction='in')
    ax.set_ylabel('Station 3')
    ax.xaxis_date()
    plt.yticks([])
    ax.grid()
    print('Plot active time for station 4...')
    ax = fig.add_subplot(514,sharex=ax1,sharey=ax1)
    ax.bar(dates,st4,width=1/24.,edgecolor='none')
    ax.tick_params(direction='in')
    ax.set_ylabel('Station 4')
    ax.xaxis_date()
    plt.yticks([])
    ax.grid(False)
    print('Plot active time for station 0...')
    ax = fig.add_subplot(515,sharex=ax1,sharey=ax1)
    ax.bar(dates,st0,width=1/24.,edgecolor='none')
    ax.tick_params(direction='in')
    ax.set_ylabel('Unnamed')
    ax.xaxis_date()
    plt.yticks([])
    ax.grid(False)
    ax.set_xlabel(r'Hourly activity in %s %i (UTC)'%(d0.strftime("%B"),y0))
    ax1.xaxis.set_major_formatter(md.DateFormatter('%d'))
    ax1.xaxis.set_major_locator(md.DayLocator())
    ax1.set_xlim(d0,d1)
    ax1.set_ylim(0,1)
    plt.savefig('%i-%02i'%(y0,m0),transparent=True,dpi=300)
    
def check24hrs(t0,t1,station):
    """
    This operation will display the active periods for which data are
    available from every sensors.

    Parameters
    ----------

    date : str
      Year and month to display activity from. The format shoud be YYYY-MM.
    """
    # Download metadata from Google Drive
    sys.stderr.write('Retrieve information from Google Drive...')
    os.system('skicka ls -r /MagneticFieldData/ > data')
    data = numpy.loadtxt('data',dtype=str,delimiter='\n')
    print(sys.stderr,' done!')
    # Get date list
    t0 = datetime(*numpy.array(t0.split('-'),dtype=int))
    t1 = datetime(*numpy.array(t1.split('-'),dtype=int))
    dt = timedelta(hours=1)
    dates = numpy.arange(t0,t1,dt)
    # List file path for each date and each station
    tick = 0
    for d in dates:
        year  = d.astype(object).year
        month = d.astype(object).month
        day   = d.astype(object).day
        hour  = d.astype(object).hour
        path  = 'MagneticFieldData/%i/%i/%i/%i/'%(year,month,day,hour)
        fname = '%i-%i-%i_%i-xx.zip'%(year,month,day,hour)
        if path+'NURI-station-%02i/'%station+fname in data:
            tick+=1
        if tick==24:
            print(year,month,day,hour)
        if hour==23:
            tick = 0
    os.system('rm data')

def download(period,date,length,station):
    """
    Download data for specific period. The default period downloaded will
    be the last 24 hours.

    Parameters
    ----------
    period : array
      Starting and ending dates of selected period
    date : str
      Starting date
    length : float
      Time length in seconds
    station : float
      Station number. Default is all stations.

    Examples
    --------
    >>> nuri download --start 2016-03-20 --end 2016-03-21 --station 4
    """
    # Convert start and end date into datetime object
    if period==None and date==None:
        print('You must defined either a period or a date and length. Abort.')
        quit()
    if period!=None:
        d0 = datetime(*numpy.array(period[0].split('-'),dtype=int))
        d1 = datetime(*numpy.array(period[1].split('-'),dtype=int))
    else:
        d0 = datetime(*numpy.array(date.split('-'),dtype=int))
        d1 = d0 + timedelta(seconds=float(length))
    # Store data for current month
    y0,m0 = d0.year,d0.month
    os.system('skicka ls -r /MagneticFieldData/%s/%s/ > data'%(y0,m0))
    data = numpy.loadtxt('data',dtype=str,delimiter='\n')
    # Loop through every hour
    for d in numpy.arange(d0,d1,timedelta(hours=1)):
        # Extract individual parameters from datetime object
        year  = d.astype(object).year
        month = d.astype(object).month
        day   = d.astype(object).day
        hour  = d.astype(object).hour
        # Define remote path to and filename of the compressed archive
        path  = 'MagneticFieldData/%i/%i/%i/%i/'%(year,month,day,hour)
        fzip  = '%i-%i-%i_%i-xx.zip'%(year,month,day,hour)
        # Loop through each station
        for i in range(4):
            # Define full path to archive
            fullpath = path+'NURI-station-%02i/'%(i+1)+fzip
            # Check if station is targeted and remote path exists
            if (station==None or int(station)==i+1) and fullpath in data:
                # Download, uncompress and store data
                os.system('mkdir -p ./NURI-station-%02i/'%(i+1))
                os.system('skicka download /%sNURI-station-%02i/%s ./'%(path,i+1,fzip))
                os.system('unzip '+fzip)
                os.system('mv %i-%i-%i_%i-xx_* NURI-station-%02i'%(year,month,day,hour,i+1))
                os.system('rm '+fzip)
    os.system('rm data')

def notification():
    '''
    Check list of files on server and notify station if missing data.
    This operation will check if station are uploading data properly
    and will notify the relevant station's owner if missing data are
    being detected.
    '''
    os.chdir(setup.home+'/analysis/nuri/activetime/')
    sys.stderr.write('Retrieve information from Google Drive...')
    os.system('skicka ls -r /MagneticFieldData/ > data')
    data = np.loadtxt('data',dtype=str,delimiter='\n')
    print(sys.stderr,' done!')
    for station in [1,2,3,4]:
        missing = []
        target  = datetime.utcnow()-timedelta(days=1)
        for hr in range(24):
            target = datetime(target.year,target.month,target.day,hr)
            str1   = target.strftime('%Y/%-m/%-d/%-H')
            str2   = target.strftime('%Y-%-m-%-d_%-H')
            if 'MagneticFieldData/%s/NURI-station-%02i/%s-xx.zip'%(str1,station,str2) not in data:
                missing.append(hr)
        if len(missing)>0:
            yesterday   = (date.today()-timedelta(1)).strftime('%A, %B %d, %Y')
            EMAIL_FROM  = 'vincentdumont11@gmail.com'
            EMAIL_TO    = ['bale@ssl.berkeley.edu'] if station==1 else \
                          ['vdumont@berkeley.edu']  if station==2 else \
                          ['wurtele@berkeley.edu']  if station==3 else \
                          ['dbudker@gmail.com']
            EMAIL_SPACE = ', '
            hrs = EMAIL_SPACE.join(['%i'%hr for hr in missing])
            msg ="Dear Station's owner,\n"
            msg+='\n'
            msg+='%i out of 24 data files are found missing from yesterday:\n'%len(missing)
            msg+='%s\n'%yesterday
            msg+='\n'
            msg+='The missing hours are the following:\n'
            msg+='%s\n'%hrs
            msg+='\n'
            msg+='Please check your station and make sure the computer is turned on\n'
            msg+='and all the USB cables are properly connected.\n'
            msg+='\n'
            msg+='Here are the instructions to reset the station:\n'
            msg+='\n'
            msg+='1. Disconnect each USB cable;\n'
            msg+='2. Shut down the computer;\n'
            msg+='3. Reconnect the cables;\n'
            msg+='4. Turn on the computer;\n'
            msg+='5. Open the data grabber application (available from the Desktop)\n'
            msg+='6. Click the "refresh" button for the GPS to make sure it identifies\n'
            msg+='the correct port;\n'
            msg+='7. Hit the "open" button. If the connection works properly, you should\n'
            msg+='see a text highlighted in green mentioning the UTC time. If the text\n'
            msg+='says "No Data", then there is a problem with the USB connection, try\n'
            msg+='re-connecting the cable on a different port and refresh.\n'
            msg+='8. Click the "refresh" button for the sensor to make sure it identifies\n'
            msg+='the correct port;\n'
            msg+='9. Hit "record", you should start seeing the data streaming on the screen;\n'
            msg+='\n'
            msg+="Do not close the application! The laptop's lid can be closed though.\n"
            msg+='As long as the computer is still turned on, data should be streaming\n'
            msg+='and uploading without any problem.\n'
            msg+='\n'
            msg+='Many thanks!\n'
            msg+='Vincent'
            msg = MIMEText(msg)
            msg['Subject'] = 'NURI Automatic Notification - Missing data from station %i...'%station
            msg['From']    = EMAIL_FROM
            msg['To']      = EMAIL_SPACE.join(EMAIL_TO)
            smtpObj = smtplib.SMTP_SSL('smtp.gmail.com', 465)
            smtpObj.login(EMAIL_FROM, 'newaknlsgkwehexe')
            smtpObj.sendmail(EMAIL_FROM, EMAIL_TO, msg.as_string())
            smtpObj.quit()
    os.system('rm data')
    



###############################################################################
# Copyright (c) 2018, Urban Magnetometry Group.
#
# Produced at the University of California at Berkeley.
#
# Written by V. Dumont (vincentdumont11@gmail.com).
#
# All rights reserved.
#
# This file is part of NURI.
#
# For details, see github.com/vincentdumont/nuri.
#
# For details about use and distribution, please read NURI/LICENSE.
###############################################################################
__version__ = '0.2.0' 
from .check24hrs import *
from .extract    import *
from .fftcomp    import *
from .makespec   import *
from .normalize  import *
from .plots      import *
from .server     import *
from .split      import *
from .timing     import *
from .timing2    import *
from .transform  import *
from .utils      import *




###############################################################################
# Copyright (c) 2018, Urban Magnetometry Group.
#
# Produced at the University of California at Berkeley.
#
# Written by V. Dumont (vincentdumont11@gmail.com).
#
# All rights reserved.
#
# This file is part of NURI.
#
# For details, see github.com/vincentdumont/nuri.
#
# For details about use and distribution, please read NURI/LICENSE.
###############################################################################
def timing2():

    rep = '%s_%s'%(setup.start,setup.end)
    os.system('mkdir -p %s'%rep)
    os.chdir(rep)
    #----------------------------------------------------------
    # Extract magnetic field data
    #----------------------------------------------------------
    # Get NURI data
    path  = '/Users/vincent/ASTRO/data/NURI/timing/run%i/station1/'%setup.run
    nuri1 = magfield(setup.start,setup.end,rep=path)
    nuri1_pulse = (max(nuri1[:,1])+min(nuri1[:,1]))/2.
    print(nuri1_pulse)
    # Get NURI data
    path  = '/Users/vincent/ASTRO/data/NURI/timing/run%i/station2/'%setup.run
    nuri2 = magfield(setup.start,setup.end,rep=path)
    nuri2_pulse = (max(nuri2[:,1])+min(nuri2[:,1]))/2.
    # Get GNOME data
    path  = '/Users/vincent/ASTRO/data/GNOME/timing/'
    gnome1 = gnomedata('Berkeley02',setup.start,setup.end,rep=path)
    gnome1_pulse = (max(gnome1[:,1])+min(gnome1[:,1]))/2.
    # Get GNOME data
    path  = '/Users/vincent/ASTRO/data/GNOME/timing/'
    gnome2 = gnomedata('Berkeley02',setup.start,setup.end,rep=path,
                      setname='Pump_PID')
    gnome2_pulse = (max(gnome2[:,1])+min(gnome2[:,1]))/2.
    #----------------------------------------------------------
    # Get start and ending timestamps
    #----------------------------------------------------------
    if setup.sample==False or setup.rescale:
        tmin = max(nuri1[0,0],nuri2[0,0],gnome1[0,0],gnome2[0,0])
        tmax = min(nuri1[-1,0],nuri2[-1,0],gnome1[-1,0],gnome2[-1,0])
    else:
        start,tmin,end,tmax = time_edge(setup.start,setup.end)
    #----------------------------------------------------------
    # Identify timestamps of every pulse for every station
    #----------------------------------------------------------
    for n in range(4):
        if n==0: magdata,pulse = nuri1,nuri1_pulse
        if n==1: magdata,pulse = nuri2,nuri2_pulse
        if n==2: magdata,pulse = gnome1,gnome1_pulse
        if n==3: magdata,pulse = gnome2,gnome2_pulse
        #below = np.where(magdata[:,1]<pulse)[0]
        #above = np.where(pulse<=magdata[:,1])[0]
        #print len(above)
        #print 'a'
        #data = []
        #for i in above:
        #    if i-1 in below: data.append(magdata[i,0])
        ##data  = [magdata[i,0] for i in above if i-1 in below]
        #print 'b'
        flag,data = 0,[]
        for i in range(1,len(magdata)-1): 
            pulse_up   = magdata[i-1,1] < pulse <= magdata[i,1]
            pulse_down = magdata[i+1,1] < pulse <= magdata[i,1]
            if tmin < magdata[i,0] and flag==0 and pulse_up:
                data.append(magdata[i,0])
                flag = 1
            if tmin < magdata[i,0] and flag==1 and pulse_down:
                flag = 0
        if n==0: station1_pulse_nuri  = data
        if n==1: station2_pulse_nuri  = data
        if n==2: station1_pulse_gnome = data
        if n==3: station2_pulse_gnome = data
    #----------------------------------------------------------
    # Calculate pulse delay between for NURI and GNOME data
    #----------------------------------------------------------
    station1_diff_nuri  = station2_diff_nuri  = np.empty((0,2))
    station1_diff_gnome = station2_diff_gnome = np.empty((0,2))
    station1_delay      = station2_delay      = np.empty((0,2))
    imax = min(len(station1_pulse_nuri),len(station1_pulse_gnome),
               len(station2_pulse_nuri),len(station2_pulse_gnome))
    for i in range(1,imax):
        x = station1_pulse_nuri[i]-tmin
        y = station1_pulse_nuri[i]-station1_pulse_nuri[i-1]
        station1_diff_nuri = np.vstack((station1_diff_nuri,[x,y]))
        x = station1_pulse_gnome[i]-tmin
        y = station1_pulse_gnome[i]-station1_pulse_gnome[i-1]
        station1_diff_gnome = np.vstack((station1_diff_gnome,[x,y]))
        x = station1_pulse_nuri[i]-tmin
        y = station1_pulse_nuri[i]-station1_pulse_gnome[i]
        station1_delay = np.vstack((station1_delay,[x,y]))
        x = station2_pulse_nuri[i]-tmin
        y = station2_pulse_nuri[i]-station2_pulse_nuri[i-1]
        station2_diff_nuri = np.vstack((station2_diff_nuri,[x,y]))
        x = station2_pulse_gnome[i]-tmin
        y = station2_pulse_gnome[i]-station2_pulse_gnome[i-1]
        station2_diff_gnome = np.vstack((station2_diff_gnome,[x,y]))
        x = station2_pulse_nuri[i]-tmin
        y = station2_pulse_nuri[i]-station2_pulse_gnome[i]
        station2_delay = np.vstack((station2_delay,[x,y]))
    #----------------------------------------------------------
    # Create x axis time label
    #----------------------------------------------------------
    tbeg = datetime.fromtimestamp(tmin)
    tend = datetime.fromtimestamp(tmax)
    sbeg = tbeg.strftime('%Y-%m-%d %H:%M:%S.%f')
    send = tend.strftime('%Y-%m-%d %H:%M:%S.%f')
    label1 = 'UTC time from %s to %s [secs]'%(sbeg,send)
    label2 = '%s-%s.png'%(int(tmin),int(tmax))
    #----------------------------------------------------------
    # Histogram of pulse separation
    #----------------------------------------------------------
    diffs = [(station1_diff_nuri[:,1]-1)*1e6,
             (station1_diff_gnome[:,1]-1)*1e6,
             (station2_diff_nuri[:,1]-1)*1e6,
             (station2_diff_gnome[:,1]-1)*1e6]
    labels = ['NURI station 1','GNOME station 1',
              'NURI station 2','GNOME station 2']
    xmin = min([min(diffs[i]) for i in range(4)])
    xmax = max([max(diffs[i]) for i in range(4)])
    fig = figure(figsize=(12,6))
    fig.suptitle('Time separation between pulses\n%s'%label1,
                 fontsize=14, fontweight='bold')
    plt.subplots_adjust(left=0.1, right=0.95,
                        bottom=0.1, top=0.9,
                        hspace=0.1, wspace=0)
    ax = plt.subplot(111,xlim=[xmin,xmax])
    ax.hist(diffs,bins=100,stacked=True,fill=True,alpha=0.4,
            edgecolor="black",range=[xmin,xmax],align='left',
            label=labels)
    legend = ax.legend(loc='upper right')
    print(np.average(diffs[0]))
    print(np.average(diffs[1]))
    print(np.average(diffs[2]))
    print(np.average(diffs[3]))
    ax.set_xlabel('Pulse delay relative to 1 second [us]')
    ax.set_yscale('log', nonposy='clip')
    savefig('histogram_pulses.png')
    clf()
    #----------------------------------------------------------
    # Plot pulses separations and delay
    #----------------------------------------------------------
    fig = figure(figsize=(15,10))
    plt.subplots_adjust(left=0.08,right=0.95,
                        bottom=0.07,top=0.95,
                        hspace=0.1,wspace=0.15)
    # Plot NURI pulse difference
    ymin,ymax = min(station1_diff_nuri[:,1]), \
                max(station1_diff_nuri[:,1])
    offset    = (ymax-ymin)/10
    ymin,ymax = ymin-offset,ymax+offset
    ax1 = subplot(321,xlim=[0,tmax-tmin])
    ax1.scatter(station1_diff_nuri[:,0],
                station1_diff_nuri[:,1],
                lw=0,s=20,c='#4e73ae')
    ax1.set_ylabel('NURI Pulse separation [secs]')
    ax1.set_ylim([ymin,ymax])
    ax1.set_title('Station 1')
    plt.setp(ax1.get_xticklabels(),visible=False)
    # Plot GNOME pulse difference
    ymin,ymax = min(station1_diff_gnome[:,1]), \
                max(station1_diff_gnome[:,1])
    offset    = (ymax-ymin)/10
    ymin,ymax = ymin-offset,ymax+offset
    ax2 = subplot(323,sharex=ax1)
    ax2.scatter(station1_diff_gnome[:,0],
                station1_diff_gnome[:,1],
                lw=0,s=20,c='#4e73ae')
    ax2.set_ylim([ymin,ymax])
    ax2.set_ylabel('GNOME Pulse separation [secs]')
    plt.setp(ax2.get_xticklabels(),visible=False)
    # Plot NURI/GNOME pulse delay
    ymin,ymax = min(station1_delay[:,1]), \
                max(station1_delay[:,1])
    offset    = (ymax-ymin)/10
    ymin,ymax = ymin-offset,ymax+offset
    ax3 = subplot(325,sharex=ax1)
    ax3.scatter(station1_delay[:,0],
                station1_delay[:,1],
                lw=0,s=20,c='#4e73ae')
    ax3.set_ylim([ymin,ymax])
    ax3.set_ylabel('Pulse delay [secs]')
    ax3.set_xlabel(label1)
    # Plot NURI pulse difference
    ymin,ymax = min(station2_diff_nuri[:,1]), \
                max(station2_diff_nuri[:,1])
    offset    = (ymax-ymin)/10
    ymin,ymax = ymin-offset,ymax+offset
    ax4 = subplot(322,sharex=ax1)
    ax4.scatter(station2_diff_nuri[:,0],
                station2_diff_nuri[:,1],
                lw=0,s=20,c='#4e73ae')
    ax4.set_ylim([ymin,ymax])
    ax4.set_ylabel('NURI Pulse separation [secs]')
    ax4.set_title('Station 2')
    plt.setp(ax4.get_xticklabels(),visible=False)
    # Plot GNOME pulse difference
    ymin,ymax = min(station2_diff_gnome[:,1]), \
                max(station2_diff_gnome[:,1])
    offset    = (ymax-ymin)/10
    ymin,ymax = ymin-offset,ymax+offset
    ax5 = subplot(324,sharex=ax1)
    ax5.scatter(station2_diff_gnome[:,0],
                station2_diff_gnome[:,1],
                lw=0,s=20,c='#4e73ae')
    ax5.set_ylim([ymin,ymax])
    ax5.set_ylabel('GNOME Pulse separation [secs]')
    ax5.set_ylim(ymin,ymax)
    plt.setp(ax5.get_xticklabels(),visible=False)
    # Plot NURI/GNOME pulse delay
    ymin,ymax = min(station2_delay[:,1]), \
                max(station2_delay[:,1])
    offset    = (ymax-ymin)/10
    ymin,ymax = ymin-offset,ymax+offset
    ax6 = subplot(326,sharex=ax1)
    ax6.scatter(station2_delay[:,0],
                station2_delay[:,1],
                lw=0,s=20,c='#4e73ae')
    ax6.set_ylim([ymin,ymax])
    ax6.set_ylabel('Pulse delay [secs]')
    ax6.set_xlim([0,tmax-tmin])
    ax6.set_xlabel(label1)
    # Save figure
    plt.savefig('pulses.png')
    plt.close()
    #----------------------------------------------------------
    # Create time series movie
    #----------------------------------------------------------
    os.system('mkdir -p video')
    tstep = 1.0 #2*tdiff
    tdiff = 0.5 #1./60.*30 #0.5
    t0,t1 = tmin-tdiff,tmin+tdiff
    while t1<min(nuri1[-1,0],nuri2[-1,0],gnome1[-1,0],gnome2[-1,0]):
        tmid = (t0+t1)/2
        fig = figure(figsize=(15,10))
        plt.subplots_adjust(left=0.1, right=0.95,
                            bottom=0.1, top=0.95,
                            hspace=0.1, wspace=0)
        # NURI1 - Identify first and last indexes
        imin = abs(nuri1[:,0]-t0).argmin()-2
        imax = abs(nuri1[:,0]-t1).argmin()+2
        # NURI1 - Plot time series
        ax1 = subplot(411)
        ax1.plot((nuri1[imin:imax,0]-tmid)*1000.,nuri1[imin:imax,1])
        ax1.set_ylim(min(nuri1[:,1]),max(nuri1[:,1]))
        ax1.set_ylim(-5,10)
        ax1.set_ylabel('NURI 1 Magnetic Field [uT]')
        plt.setp(ax1.get_xticklabels(),visible=False)
        # NURI2 - Identify first and last indexes
        imin = abs(nuri2[:,0]-t0).argmin()-2
        imax = abs(nuri2[:,0]-t1).argmin()+2
        # NURI2 - Plot time series
        ax2 = subplot(412,sharex=ax1)
        ax2.plot((nuri2[imin:imax,0]-tmid)*1000.,nuri2[imin:imax,1])
        ax2.set_ylim(min(nuri2[:,1]),max(nuri2[:,1]))
        ax2.set_ylabel('NURI 2 Magnetic Field [uT]')
        plt.setp(ax2.get_xticklabels(),visible=False)
        # GNOME1 - Identify first and last indexes
        imin = abs(gnome1[:,0]-t0).argmin()-2
        imax = abs(gnome1[:,0]-t1).argmin()+2
        # GNOME1 - Plot time series
        ax3 = subplot(413,sharex=ax1)
        ax3.plot((gnome1[imin:imax,0]-tmid)*1000.,gnome1[imin:imax,1])
        ax3.set_ylim(min(gnome1[:,1]),max(gnome1[:,1]))
        ax3.set_ylabel('GNOME 1 Magnetic Field [uT]')
        plt.setp(ax3.get_xticklabels(),visible=False)
        # GNOME2 - Identify first and last indexes
        imin = abs(gnome2[:,0]-t0).argmin()-2
        imax = abs(gnome2[:,0]-t1).argmin()+2
        # GNOME2 - Plot time series
        ax4 = subplot(414,sharex=ax1)
        ax4.plot((gnome2[imin:imax,0]-tmid)*1000.,gnome2[imin:imax,1])
        ax4.set_ylim(min(gnome2[:,1]),max(gnome2[:,1]))
        ax4.set_ylabel('GNOME 2 Magnetic Field [uT]')
        # Set x axis limit and label
        ax4.set_xlim((t0-tmid)*1000.,(t1-tmid)*1000.) 
        date = datetime.fromtimestamp(tmid).strftime('%Y-%m-%d %H:%M:%S.%f')
        ax4.set_xlabel('Time series around %s UTC time [ms]'%(date))
        # Save and close figure
        plt.savefig('video/%s-%s.png'%(t0,t1))
        plt.close()
        t0 += tstep
        t1 += tstep



###############################################################################
# Copyright (c) 2018, Urban Magnetometry Group.
#
# Produced at the University of California at Berkeley.
#
# Written by V. Dumont (vincentdumont11@gmail.com).
#
# All rights reserved.
#
# This file is part of NURI.
#
# For details, see github.com/vincentdumont/nuri.
#
# For details about use and distribution, please read NURI/LICENSE.
###############################################################################

def transform():
    
    if '--help' in sys.argv or '-h' in sys.argv:
        
        print("""
              "-------------------------------------------------------------------------"
              ""
              "description:"
              ""
              "  This operation will plot the Fourier transform of given time series."
              "  The time series and FFT are shown interactively so the user can select"
              "  a desired time slot and get the FFT from that region."
              ""
              "required arguments:"
              ""
              "   --start          Starting date to retrieve data from."
              "   --end            Ending date to retrieve data from."
              "   --station        Station name"
              ""
              "optional arguments:"
              ""
              "   --discard        Discard first selected seconds"
              "   --fmin           Minimum frequency to display"
              "   --fmax           Maximum frequency to display"
              "   --nogps          No GPS provided, create time array using sample rate"
              "   --orientation    Orientation to extract if single (all by default)"
              "   --path           Custom repository where data are stored."
              "   --sample         Specify the data to be field sample, not regular."
              "   --scale          Rescale y axis"
              "   --unit           Time unit for plot."
              ""
              "example:"
              ""
              "  nuri transform --start 2017-03-09-15-58 --end 2017-03-10 \ "
              "                 --path /Users/vincent/ASTRO/data/others/sample1_background_noise/ \ "
              "                 --station magnet --sample --unit secs --nogps \ "
              "                 --discard 1 --fmin 1 --down 12 --seaborn"
              ""
              "-------------------------------------------------------------------------"
              """)
        quit()
        
    data = magfield(setup.start,setup.end,rep=setup.path)
    # Get start and ending timestamps
    if setup.start!=None and setup.end!=None:
        start,tmin,end,tmax = time_edge(setup.start,setup.end) 
    else:
        tmin,tmax = data[0,0],data[-1,0]
    # Convert every timing points to scale (hr,min,sec) units
    s = 1. if setup.unit=='secs' else 60. if setup.unit=='mins' else 3600.
    data[:,0] = (data[:,0]-data[0,0])/s
    # Create x axis time label
    tbeg = datetime.fromtimestamp(tmin)
    tend = datetime.fromtimestamp(tmax)
    sbeg = tbeg.strftime('%Y-%m-%d %H:%M:%S.%f')
    send = tend.strftime('%Y-%m-%d %H:%M:%S.%f')
    label1 = 'UTC time from %s to %s [%s]'%(sbeg,send,setup.unit)
    label2 = '%s-%s.png'%(int(tmin),int(tmax))
    # Initialise axis
    fig = figure(figsize=(15,10))
    plt.subplots_adjust(left=0.07,right=0.95,
                        bottom=0.1,top=0.95,
                        hspace=0.2,wspace=0.1)
    # Plot time series
    ax1 = subplot(231)
    ax1.plot(data[:,0],data[:,1]-np.average(data[:,1]))
    ax1.set_ylabel('Magnetic field [uT]')
    ax1.set_title('X field')
    ax2 = subplot(232,sharex=ax1,sharey=ax1)
    ax2.plot(data[:,0],data[:,2]-np.average(data[:,2]))
    ax2.set_xlabel(label1)
    ax2.set_title('Y field')
    plt.setp(ax2.get_yticklabels(),visible=False)
    ax3 = subplot(233,sharex=ax1,sharey=ax1)
    ax3.plot(data[:,0],data[:,3]-np.average(data[:,3]))
    ax3.set_xlim([data[0,0],data[-1,0]])
    ax3.set_title('Z field')
    plt.setp(ax3.get_yticklabels(),visible=False)
    # Create FFT of data
    N = len(data)
    T = 1.0 / (setup.rate/setup.down)
    f = np.linspace(0.0, 1.0/(2.0*T), N/2)
    i1 = abs(f-setup.fmin).argmin()
    i2 = len(f) if setup.fmax==None else abs(f-setup.fmax).argmin()
    f = f[i1:i2]
    x,y,z = data[:,1],data[:,2],data[:,3]
    x = 2.0/N * np.abs(fft(x)[:N//2])[i1:i2]
    y = 2.0/N * np.abs(fft(y)[:N//2])[i1:i2]
    z = 2.0/N * np.abs(fft(z)[:N//2])[i1:i2]
    # Plot FFT for x direction
    ax4 = subplot(234)
    ax4.plot(f,x,'k')
    ax4.set_ylabel('|Y(f)|')
    # Plot FFT for y direction
    ax5 = subplot(235,sharex=ax4,sharey=ax4)
    ax5.plot(f,y,'k')
    ax5.set_xlabel('Frequency [Hz]')
    plt.setp(ax5.get_yticklabels(),visible=False)
    # Plot FFT for z direction
    ax6 = subplot(236,sharex=ax4,sharey=ax4)
    ax6.plot(f,z,'k')
    ax6.set_xlim(1,f[-1])
    ax6.set_ylim(0,max(np.hstack((x,y,z)))/setup.scale)
    plt.setp(ax6.get_yticklabels(),visible=False)
    # Save figure
    plt.savefig(label2)
    plt.close(fig)



import numpy,scipy
import matplotlib.pyplot as plt

class normalize():
    '''
    Quick & Dirty manual normalisation using spline interpolation.
    '''

    def __init__(self,data,step=3600*24,saved=None,play=True):

        self.data = data
            
        if saved!=None:
            self.points = numpy.loadtxt(saved,dtype='float')
        else:
            delta  = int((self.data.times.value[-1]-self.data.times.value[0])/step)
            points = numpy.array([[self.data.times.value[0]+i*step,numpy.median(self.data.value)] for i in range(delta+1)])
            self.points = numpy.vstack((points,[self.data.times.value[-1],numpy.median(self.data.value)]))

        self.co = scipy.interpolate.interp1d(self.points[:,0],self.points[:,1],kind='cubic')(self.data.times.value)

        if play:
            self.fig = plt.figure()
            plt.subplots_adjust(left=0.05, right=0.95, bottom=0.02, top=0.95, hspace=0.2, wspace=0)
            ax1 = plt.subplot2grid((4,1), (0, 0), rowspan=3)
            ax1.set_xlim(self.data.times.value[0],self.data.times.value[-1])
            ax1.plot(self.data.times.value,self.data.value,color='black',lw=0.3,zorder=2)
            ax1.axhline(y=numpy.median(self.data.value),color='grey',ls='dashed',zorder=2,lw=2,alpha=0.7)
            self.cont1  = ax1.scatter(self.points[:,0],self.points[:,1],marker='x',color='red',zorder=4)
            self.cont2, = ax1.plot(self.data.times.value,self.co,color='red',lw=1,zorder=3)
            self.ax1 = ax1
            ax2 = plt.subplot2grid((4,1), (3, 0),sharex=ax1)
            ax2.set_xlim(self.data.times.value[0],self.data.times.value[-1])
            self.norm, = ax2.plot(self.data.times.value,self.data.value/self.co,color='black',lw=0.3,zorder=1)
            self.ax2 = ax2
            self.fig.canvas.mpl_connect('key_press_event', self.press)
            plt.show()

        data.value[:] = (data.value / self.co) * numpy.median(data.value)
        self.data = data
        
    def press(self,event):
    
        if event.key in ['a','d',' ']:
            self.pts_edit(event)
            self.fitcont(event)
            self.fig.canvas.draw()
        if event.key=='s':
            print('|- Points and normalised spectrum saved!')
            outpts = open('saved.dat','w')
            for i in range (len(self.points)):
                sample = "{times:>15}{field:>20}".format(times=self.points[i,0],field=self.points[i,1])
                outpts.write(sample+'\n')
            outpts.close()
        if event.key=='q':
            print('|- Close window')
            plt.close()

    def pts_edit(self,event):

        if event.key=='a':
            self.points = numpy.vstack((self.points,[event.xdata,event.ydata]))
            self.points = self.points[self.points[:,0].argsort()]
        if event.key=='d':
            i = abs(self.points[:,0]-event.xdata).argmin()
            self.points = numpy.delete(self.points,i,axis=0)

    def fitcont(self,event):
    
        i = abs(self.points[:,0]-event.xdata).argmin()
        x = self.data.times.value[0] if i==0 else self.data.times.value[-1] if i==len(self.points)-1 else event.xdata
        self.points[i] = [x,event.ydata]
        self.co = scipy.interpolate.interp1d(self.points[:,0],self.points[:,1],kind='cubic')(self.data.times.value)
        self.cont1.remove()
        self.cont1 = self.ax1.scatter(self.points[:,0],self.points[:,1],marker='x',color='red',zorder=4)
        self.cont2.remove()
        self.cont2, = self.ax1.plot(self.data.times.value,self.co,color='red',lw=1,zorder=3)
        self.norm.remove()
        self.norm, = self.ax2.plot(self.data.times.value,self.data.value/self.co,color='black',lw=0.3,zorder=1)

    def get_updated_data(self):
        return self.data



#!/usr/bin/env python2
import argparse,datetime,math,matplotlib,mlpy,numpy,os,struct,sys
import matplotlib.pyplot as plt
from datetime          import datetime
from gwpy.plotter      import SpectrogramPlot
from gwpy.timeseries   import TimeSeries
from matplotlib.ticker import LogLocator
from scipy             import fftpack,signal

def plot_spectrogram(data,tmin=None,tmax=None,fmin=None,fmax=None,vmin=None,vmax=None,
                     mmax=None,mmin=None,mode='wavelet',omega0=6,dj=0.05,fct='morlet',
                     stride=None,nfft=None,overlap=None,scale='log',
                     funit='Hz',tunit='secs',cmap='inferno',zone='Local',fname=None):
    """
    Plot multiplot figure with time series, PSD and spectrogram.

    Parameters
    ----------
    data : TimeSeries
      Magnetic field data
    tmin, tmax : datetime
      First and last timestamps
    fmin, fmax : float
      Minimum and maximum frequencies
    vmin, vmax : float
      Minimum and maximum color values
    mode : str
      Spectrogram mode, wavelet or Fourier. Default is Fourier
    omega0 : int
      Wavelet function parameter
    dj : float
      Scale resolution (smaller values of dj give finer resolution)
    fct : str
      Wavelet function (morlet,paul,dog)
    stride : float
      Length of segment
    nfft : float
      Length of FFT
    overlap : float
      Length of overlapping segment
    cmap : str
      Colormap
    scale : str
      Plotted frequency scale. Default is "log".
    funit : strg
      Frequency unit, Hz or mHz. Default is Hz.
    tunit : str
      Time unit, secs, mins or hrs. Default is mins.
    fname : str
      Output file name.
    
    Notes
    -----
    The `matplotlib.pyplot.imshow <https://matplotlib.org/api/pyplot_api.html?highlight=matplotlib%20pyplot%20imshow#matplotlib.pyplot.imshow>`_ module is
    used to plot the wavelet spectrogram. This module is usually used
    to plot raw images and assumes that the position of the cell in the
    input spectrogram array directly represents the position of the pixel
    in the raw image. That is, for an input Python array (in which rows
    are appended below previous ones), the first row in the array is
    assumed to represent the top line of pixel in the image. Therefore,
    in order to plot the spectrogram array using the imshow module, one
    needs to carefully check that the rows (which are representative of
    the frequency bands), are stored in descending order such that the
    lowest frequency is placed at the end (bottom) of the array.
    """
    if mode=='wavelet' and scale=='linear':
        print('Warning: Wavelet mode chosen. Scale will be changed to log.')
        scale = 'log'
    # Initialise figure
    fig = plt.figure(figsize=(24,14),frameon=False)
    plt.subplots_adjust(left=0.07, right=0.95, bottom=0.1, top=0.95, hspace=0, wspace=0)
    ax1 = fig.add_axes([0.20,0.75,0.683,0.20])
    ax2 = fig.add_axes([0.20,0.10,0.683,0.64], sharex=ax1)
    ax3 = fig.add_axes([0.07,0.10,0.123,0.64])
    ax4 = fig.add_axes([0.89,0.10,0.030,0.64])
    # Prepare timing range
    tmin = data.times[0].value  if tmin==None else tmin
    tmax = data.times[-1].value if tmax==None else tmax
    mask = (data.times.value>=tmin) & (data.times.value<=tmax)
    scale_factor = 3600. if tunit=='hrs' else 60. if tunit=='mins' else 1
    times = (data[mask].times.value-tmin)/scale_factor
    dt = 1./data.sample_rate.value
    # Plot time series
    ax1.plot(times,data[mask].value,alpha=0.5)
    ax1.set_ylabel('Magnetic Fields [uT]',fontsize=11)
    ax1.tick_params(bottom='off',labelbottom='off')
    if mmin!=None: ax1.set_ylim(ymin=mmin)
    if mmax!=None: ax1.set_ylim(ymax=mmax)
    ax1.set_xlim(0,(tmax-tmin)/scale_factor)
    ax1.grid(b=True, which='major', alpha=0.7, ls='--')
    if mode=='wavelet':
        # Calculate wavelet parameters
        scales = mlpy.wavelet.autoscales(N=len(data[mask].value),dt=dt,dj=dj,wf=fct,p=omega0)
        spec = mlpy.wavelet.cwt(data[mask].value,dt=dt,scales=scales,wf=fct,p=omega0)
        freq = (omega0 + numpy.sqrt(2.0 + omega0 ** 2)) / (4 * numpy.pi * scales[1:])
        freq = freq * 1000. if funit=='mHz' else freq
        spec = numpy.abs(spec)**2
        spec = spec[::-1]
        # Define minimum and maximum frequencies
        fmin_log,fmax_log = min(freq),max(freq)
        fmin_linear,fmax_linear = min(freq),max(freq)
        if fmin!=None:
            log_ratio = (numpy.log10(fmin)-numpy.log10(min(freq)))/(numpy.log10(max(freq))-numpy.log10(min(freq)))
            fmin_linear = min(freq)+log_ratio*(max(freq)-min(freq))
            fmin_log = fmin
        if fmax!=None:
            log_ratio = (numpy.log10(fmax)-numpy.log10(min(freq)))/(numpy.log10(max(freq))-numpy.log10(min(freq)))
            fmax_linear = min(freq)+log_ratio*(max(freq)-min(freq))
            fmax_log = fmax
        # Get minimum and maximum amplitude in selected frequency range
        idx = numpy.where(numpy.logical_and(fmin_log<freq[::-1],freq[::-1]<fmax_log))[0]
        vmin = vmin if vmin!=None else numpy.sort(numpy.unique(spec[idx]))[1]
        vmax = spec[idx].max() if vmax==None else vmax
        # Plot spectrogram
        img = ax2.imshow(spec,extent=[times[0],times[-1],freq[-1],freq[0]],aspect='auto',
                        interpolation='nearest',cmap=cmap,norm=matplotlib.colors.LogNorm(vmin,vmax)) 
        ax2.set_xlabel('Time [%s] from %s %s (%s)'%(tunit,datetime.utcfromtimestamp(tmin),zone,tmin),fontsize=15)
        ax2.set_xlim(0,(tmax-tmin)/scale_factor)
        ax2.set_yscale('linear')
        ax2.set_ylim(fmin_linear,fmax_linear)
        ax2.grid(False)
        # Set up axis range for spectrogram
        twin_ax = ax2.twinx()
        twin_ax.set_yscale('log')
        twin_ax.set_xlim(0,(tmax-tmin)/scale_factor)
        twin_ax.set_ylim(fmin_log,fmax_log)
        twin_ax.spines['top'].set_visible(False)
        twin_ax.spines['right'].set_visible(False)
        twin_ax.spines['bottom'].set_visible(False)
        ax2.tick_params(which='both', labelleft=False, left=False)
        twin_ax.tick_params(which='both', labelleft=False,left=False, labelright=False, right=False)
        twin_ax.grid(False)
    if mode=='fourier':
        freq, times, spec = signal.spectrogram(data[mask],fs=data.sample_rate.value,
                                           nperseg=stride,noverlap=overlap,nfft=nfft)
        # Convert time array into minute unit
        times = (numpy.linspace(data[mask].times.value[0],data[mask].times.value[-1],len(times))-tmin)/scale_factor
        # Define minimum and maximum frequencies
        freq = freq * 1000. if funit=='mHz' else freq
        fmin = freq[1]      if fmin==None    else fmin
        fmax = max(freq)    if fmax==None    else fmax
        fmin_log,fmax_log = fmin,fmax
        # Get minimum and maximum amplitude in selected frequency range
        idx = numpy.where(numpy.logical_and(fmin<=freq,freq<=fmax))[0]
        vmin = vmin if vmin!=None else numpy.sort(numpy.unique(spec[idx]))[1]
        vmax = spec[idx].max() if vmax==None else vmax
        # Plot spectrogram
        img = ax2.pcolormesh(times,freq,spec,cmap=cmap,norm=matplotlib.colors.LogNorm(vmin,vmax))
        ax2.set_xlabel('Time [%s] from %s %s (%s)'%(tunit,datetime.utcfromtimestamp(tmin),zone,tmin),fontsize=15)
        ax2.set_xlim(0,(tmax-tmin)/scale_factor)
        ax2.set_ylim(fmin,fmax)
        ax2.set_yscale(scale)
        ax2.set_ylabel('Frequency [%s]'%funit,fontsize=15,labelpad=40)
        ax2.tick_params(which='both', labelleft=False, left=False)
        ax2.grid(False)
    # Calculate Power Spectral Density
    N = len(data[mask].value)
    delta_t = 1/data.sample_rate.value
    delta_f = 1. / (N * delta_t)
    f = delta_f * numpy.arange(N / 2)
    f = f * 1000. if funit=='mHz' else f
    PSD = abs(delta_t * fftpack.fft(data[mask].value)[:N / 2]) ** 2
    psd = numpy.vstack((f,PSD)).T
    # Plot Power Spectral Density
    ticks = matplotlib.ticker.FuncFormatter(lambda v,_:("$10^{%.0f}$"%math.log(v,10)))
    ax3.loglog(psd[:,1],psd[:,0],alpha=0.5)
    ax3.invert_xaxis()
    ax3.set_ylim(fmin_log,fmax_log)
    ax3.set_ylabel('Frequency [%s]'%funit,fontsize=15)
    ax3.set_xlabel('PSD',fontsize=15)
    ax3.grid(b=True, which='major', alpha=0.7, ls='--')
    # Add color bar and save figure
    cb = fig.colorbar(img,cax=ax4)
    cb.set_ticks(LogLocator())
    cb.set_clim(vmin,vmax)
    ax4.set_ylabel('Power $|\mathrm{W}_v|^2$ $[\mu T^2/\mathrm{Hz}]$',fontsize=15)
    plt.show() if fname==None else plt.savefig(fname)#,frameon=False)
    plt.close(fig)

# Extract input arguments
parser = argparse.ArgumentParser(prog='vmr_wavelet',description='Make wavelet from VMR data.')
parser.add_argument("-f","--logfile",help='Log filename: log.bin',required=True)
parser.add_argument("-r","--rate", default=None,type=float,help='Targeted sampling rate')
parser.add_argument("-o","--out", default='wavelet.png',help='Output figure name')
parser.add_argument("--tmin", default=None,type=float,help='Start timestamp')
parser.add_argument("--tmax", default=None,type=float,help='End timestamp')
parser.add_argument("--mmin", default=None,type=float,help='Minimum magnetic field')
parser.add_argument("--mmax", default=None,type=float,help='Maximum magnetic field')
parser.add_argument("--ufactor", default=1.,type=float,help='Factor from local to uT')
args = parser.parse_args()
# Read binary file and store floating values in numpy array
with open(args.logfile,'rb') as f:
  data = f.read()
  size = int(len(data)/32)
  data = struct.unpack('dddd'*size,data)
  data = numpy.reshape(data,(size,4))
# Identify samples to discard
if args.tmin!=None:
  idxs = numpy.where(data[:,0]<args.tmin)[0]
  data = numpy.delete(data,idxs,axis=0)
if args.tmax!=None:
  idxs = numpy.where(data[:,0]>args.tmax)[0]
  data = numpy.delete(data,idxs,axis=0)
# Check targeted sampling rate if mentioned
sample_rate = int(1/numpy.average([data[i+1,0]-data[i,0] for i in range(len(data)-1)]))
target_rate = float(sample_rate) if args.rate==None else float(args.rate)
downsampling = sample_rate/target_rate
if (downsampling).is_integer()==False:
  print("ERROR: Data sampling rate (%i) not divisible by targeted rate (%i)"%(sample_rate,target_rate))
  quit()
# Print out first and last timestamp values
times = data[:,0]
date0 = datetime.fromtimestamp(times[0]).strftime('%Y-%m-%d %H:%M:%S.%f')
date1 = datetime.fromtimestamp(times[-1]).strftime('%Y-%m-%d %H:%M:%S.%f')
print(len(data),'data points were streamed from %s to %s'%(date0,date1))
# Calculate scalar value of magnetic field for each sample
scalar = numpy.sqrt(numpy.sum(abs(data[:,1:]/args.ufactor)**2,axis=1))
scalar = signal.resample(scalar,int(len(scalar)/downsampling))
print('Data are resampled by a factor',downsampling,'from',sample_rate,'down to',target_rate,'sample/sec.')
# Store data into GwPy time series format
data = TimeSeries(scalar,sample_rate=target_rate,epoch=times[0])
# Plot wavelet spectrogram
tmin = times[0]  if args.tmin==None else args.tmin
tmax = times[-1] if args.tmax==None else args.tmax
mmin = min(data.value) if args.mmin==None else args.mmin
mmax = max(data.value) if args.mmax==None else args.mmax
name = 'wavelet_%i-%i'%(tmin,tmax) if args.out==None else args.out
plot_spectrogram(data,fname=name,tunit='mins',tmin=tmin,tmax=tmax,mmin=mmin,mmax=mmax)



###############################################################################
# Copyright (c) 2018, Urban Magnetometry Group.
#
# Produced at the University of California at Berkeley.
#
# Written by V. Dumont (vincentdumont11@gmail.com).
#
# All rights reserved.
#
# This file is part of NURI.
#
# For details, see github.com/vincentdumont/nuri.
#
# For details about use and distribution, please read NURI/LICENSE.
###############################################################################
import nuri,os,math,numpy,matplotlib,mlpy,glob
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from datetime          import datetime,timedelta
from gwpy.timeseries   import TimeSeries
from matplotlib.ticker import LogLocator
from scipy             import fftpack,signal

# Clean customized matplotlib settings from GwPy
matplotlib.rcParams.update(matplotlib.rcParamsDefault)
# Use seaborn plotting style
plt.style.use('seaborn')
# Set new default label size
plt.rc('font', size=14, family='sans-serif')
plt.rc('axes', labelsize=14, linewidth=0.2)
plt.rc('legend', fontsize=12, handlelength=10)
plt.rc('xtick', labelsize=10)
plt.rc('ytick', labelsize=10)
# Increase chunk size
plt.rcParams['agg.path.chunksize'] = 10000

def plot_spectrogram(data,tmin=None,tmax=None,fmin=None,fmax=None,vmin=None,vmax=None,
                     mmax=None,mmin=None,mode='wavelet',omega0=6,dj=0.05,fct='morlet',
                     stride=None,nfft=None,overlap=None,scale='log',median=False,
                     funit='Hz',tunit='secs',cmap='inferno',zone='Local',fname=None):
    """
    Plot multiplot figure with time series, PSD and spectrogram.

    Parameters
    ----------
    data : TimeSeries
      Magnetic field data
    tmin, tmax : datetime
      First and last timestamps
    fmin, fmax : float
      Minimum and maximum frequencies
    vmin, vmax : float
      Minimum and maximum color values
    mode : str
      Spectrogram mode, wavelet or Fourier. Default is Fourier
    omega0 : int
      Wavelet function parameter
    dj : float
      Scale resolution (smaller values of dj give finer resolution)
    fct : str
      Wavelet function (morlet,paul,dog)
    stride : float
      Length of segment
    nfft : float
      Length of FFT
    overlap : float
      Length of overlapping segment
    cmap : str
      Colormap
    scale : str
      Plotted frequency scale. Default is "log".
    funit : strg
      Frequency unit, Hz or mHz. Default is Hz.
    tunit : str
      Time unit, secs, mins or hrs. Default is mins.
    fname : str
      Output file name.
    
    Notes
    -----
    The `matplotlib.pyplot.imshow <https://matplotlib.org/api/pyplot_api.html?highlight=matplotlib%20pyplot%20imshow#matplotlib.pyplot.imshow>`_ module is
    used to plot the wavelet spectrogram. This module is usually used
    to plot raw images and assumes that the position of the cell in the
    input spectrogram array directly represents the position of the pixel
    in the raw image. That is, for an input Python array (in which rows
    are appended below previous ones), the first row in the array is
    assumed to represent the top line of pixel in the image. Therefore,
    in order to plot the spectrogram array using the imshow module, one
    needs to carefully check that the rows (which are representative of
    the frequency bands), are stored in descending order such that the
    lowest frequency is placed at the end (bottom) of the array.
    """
    if mode=='wavelet' and scale=='linear':
        print('Warning: Wavelet mode chosen. Scale will be changed to log.')
        scale = 'log'
    # Initialise figure
    fig = plt.figure(figsize=(24,14),frameon=False)
    plt.subplots_adjust(left=0.07, right=0.95, bottom=0.1, top=0.95, hspace=0, wspace=0)
    ax1 = fig.add_axes([0.20,0.75,0.683,0.20])
    ax2 = fig.add_axes([0.20,0.10,0.683,0.64], sharex=ax1)
    ax3 = fig.add_axes([0.07,0.10,0.123,0.64])
    ax4 = fig.add_axes([0.89,0.10,0.030,0.64])
    # Prepare timing range
    tmin = data.times[0].value  if tmin==None else tmin
    tmax = data.times[-1].value if tmax==None else tmax
    mask = (data.times.value>=tmin) & (data.times.value<=tmax)
    scale_factor = 3600. if tunit=='hrs' else 60. if tunit=='mins' else 1
    times = (data[mask].times.value-tmin)/scale_factor
    dt = 1./data.sample_rate.value
    # Plot time series
    median = numpy.median(data[mask].value) if median else 0.
    ax1.plot(times,data[mask].value-median,alpha=0.5)
    ax1.set_ylabel('Magnetic Fields [uT]',fontsize=11)
    ax1.tick_params(bottom='off',labelbottom='off')
    if mmin!=None: ax1.set_ylim(ymin=mmin)
    if mmax!=None: ax1.set_ylim(ymax=mmax)
    ax1.set_xlim(0,(tmax-tmin)/scale_factor)
    ax1.grid(b=True, which='major', alpha=0.7, ls='--')
    if mode=='wavelet':
        # Calculate wavelet parameters
        scales = mlpy.wavelet.autoscales(N=len(data[mask].value),dt=dt,dj=dj,wf=fct,p=omega0)
        spec = mlpy.wavelet.cwt(data[mask].value,dt=dt,scales=scales,wf=fct,p=omega0)
        freq = (omega0 + numpy.sqrt(2.0 + omega0 ** 2)) / (4 * numpy.pi * scales[1:])
        freq = freq * 1000. if funit=='mHz' else freq
        spec = numpy.abs(spec)**2
        spec = spec[::-1]
        # Define minimum and maximum frequencies
        fmin_log,fmax_log = min(freq),max(freq)
        fmin_linear,fmax_linear = min(freq),max(freq)
        if fmin!=None:
            log_ratio = (numpy.log10(fmin)-numpy.log10(min(freq)))/(numpy.log10(max(freq))-numpy.log10(min(freq)))
            fmin_linear = min(freq)+log_ratio*(max(freq)-min(freq))
            fmin_log = fmin
        if fmax!=None:
            log_ratio = (numpy.log10(fmax)-numpy.log10(min(freq)))/(numpy.log10(max(freq))-numpy.log10(min(freq)))
            fmax_linear = min(freq)+log_ratio*(max(freq)-min(freq))
            fmax_log = fmax
        # Get minimum and maximum amplitude in selected frequency range
        idx = numpy.where(numpy.logical_and(fmin_log<freq[::-1],freq[::-1]<fmax_log))[0]
        vmin = vmin if vmin!=None else numpy.sort(numpy.unique(spec[idx]))[1]
        vmax = spec[idx].max() if vmax==None else vmax
        # Plot spectrogram
        img = ax2.imshow(spec,extent=[times[0],times[-1],freq[-1],freq[0]],aspect='auto',
                        interpolation='nearest',cmap=cmap,norm=matplotlib.colors.LogNorm(vmin,vmax)) 
        ax2.set_xlabel('Time [%s] from %s to %s %s Time (%s - %s)'%(tunit,datetime.utcfromtimestamp(tmin),datetime.utcfromtimestamp(tmax),zone,tmin,tmax),fontsize=15)
        ax2.set_xlim(0,(tmax-tmin)/scale_factor)
        ax2.set_yscale('linear')
        ax2.set_ylim(fmin_linear,fmax_linear)
        ax2.grid(False)
        # Set up axis range for spectrogram
        twin_ax = ax2.twinx()
        twin_ax.set_yscale('log')
        twin_ax.set_xlim(0,(tmax-tmin)/scale_factor)
        twin_ax.set_ylim(fmin_log,fmax_log)
        twin_ax.spines['top'].set_visible(False)
        twin_ax.spines['right'].set_visible(False)
        twin_ax.spines['bottom'].set_visible(False)
        ax2.tick_params(which='both', labelleft=False, left=False)
        twin_ax.tick_params(which='both', labelleft=False,left=False, labelright=False, right=False)
        twin_ax.grid(False)
    if mode=='fourier':
        freq, times, spec = signal.spectrogram(data[mask],fs=data.sample_rate.value,
                                           nperseg=stride,noverlap=overlap,nfft=nfft)
        # Convert time array into minute unit
        times = (numpy.linspace(data[mask].times.value[0],data[mask].times.value[-1],len(times))-tmin)/scale_factor
        # Define minimum and maximum frequencies
        freq = freq * 1000. if funit=='mHz' else freq
        fmin = freq[1]      if fmin==None    else fmin
        fmax = max(freq)    if fmax==None    else fmax
        fmin_log,fmax_log = fmin,fmax
        # Get minimum and maximum amplitude in selected frequency range
        idx = numpy.where(numpy.logical_and(fmin<=freq,freq<=fmax))[0]
        vmin = vmin if vmin!=None else numpy.sort(numpy.unique(spec[idx]))[1]
        vmax = spec[idx].max() if vmax==None else vmax
        # Plot spectrogram
        img = ax2.pcolormesh(times,freq,spec,cmap=cmap,norm=matplotlib.colors.LogNorm(vmin,vmax))
        ax2.set_xlabel('Time [%s] from %s to %s %s Time (%s)'%(tunit,datetime.utcfromtimestamp(tmin),datetime.utcfromtimestamp(tmax),zone,tmin),fontsize=15)
        ax2.set_xlim(0,(tmax-tmin)/scale_factor)
        ax2.set_ylim(fmin,fmax)
        ax2.set_yscale(scale)
        ax2.set_ylabel('Frequency [%s]'%funit,fontsize=15,labelpad=40)
        ax2.tick_params(which='both', labelleft=False, left=False)
        ax2.grid(False)
    # Calculate Power Spectral Density
    N = len(data[mask].value)
    delta_t = 1/data.sample_rate.value
    delta_f = 1. / (N * delta_t)
    f = delta_f * numpy.arange(N / 2)
    f = f * 1000. if funit=='mHz' else f
    PSD = abs(delta_t * fftpack.fft(data[mask].value)[:N / 2]) ** 2
    psd = numpy.vstack((f,PSD)).T
    # Plot Power Spectral Density
    ticks = matplotlib.ticker.FuncFormatter(lambda v,_:("$10^{%.0f}$"%math.log(v,10)))
    ax3.loglog(psd[:,1],psd[:,0],alpha=0.5)
    ax3.invert_xaxis()
    ax3.set_ylim(fmin_log,fmax_log)
    ax3.set_ylabel('Frequency [%s]'%funit,fontsize=15)
    ax3.set_xlabel('PSD',fontsize=15)
    ax3.grid(b=True, which='major', alpha=0.7, ls='--')
    # Add color bar and save figure
    cb = fig.colorbar(img,cax=ax4)
    cb.set_ticks(LogLocator())
    cb.set_clim(vmin,vmax)
    ax4.set_ylabel('Power $|\mathrm{W}_v|^2$ $[\mu T^2/\mathrm{Hz}]$',fontsize=15)
    plt.show() if fname==None else plt.savefig(fname)#,frameon=False)
    plt.close(fig)

def plot_time_series(data,tmin=None,tmax=None,ymin=None,ymax=None,regions=[],
                     fname='time_series',zone='Local',tunit='secs',tbs=False):
    """
    Plot time series.

    Parameters
    ----------
    data : gwpy.timeseries.TimeSeries
      Time series data
    tmin : datetime
      First timestamp
    tmax : datetime
      Last timestamp
    fname : str
      Filename
    zone : str
      Output time zone, either UTC or Local
    tunit : str
      Scale of time axis (hrs, mins, secs)
    tbs : bool
      To Be Saved flag
    regions : list
      List of time regions to colorize
    """
    # Define timestamps if not defined by user
    tmin = data.times[0].value  if tmin==None else tmin
    tmax = data.times[-1].value if tmax==None else tmax
    # Re-determine the mask in case timestamps are defined
    mask = (data.times.value>=tmin) & (data.times.value<=tmax)
    # Estimate scale factor for time axis
    scale_factor = 3600. if tunit=='hrs' else 60. if tunit=='mins' else 1
    # Do the plotting
    fig = plt.figure(figsize=(12,5))
    plt.plot((data.times.value[mask]-tmin)/scale_factor,data.value[mask])
    if len(regions)>0:
        for text,tbeg,span in regions:
            tbeg = (str2timestamp(tbeg)-tmin)/scale_factor
            tend = tbeg+span/scale_factor
            plt.axvspan(tbeg,tend,color='red',ls='dotted',alpha=0.1)
            plt.text(tbeg+(tend-tbeg)/2,max(data.value[mask]),'%s'%text,fontsize=25,color='red',ha='center')
    plt.xlabel('Time [%s] from %s %s (%i)'%(tunit,datetime.utcfromtimestamp(tmin),zone,tmin))
    plt.ylabel('Magnetic Field [$\mathrm{\mu}$T]')
    plt.xlim(0,(tmax-tmin)/scale_factor)
    if ymin!=None: plt.ylim(ymin=ymin)
    if ymax!=None: plt.ylim(ymax=ymax)
    plt.tight_layout()
    plt.savefig('%s.png'%fname,frameon=False,transparent=True) if tbs else plt.show()
    plt.close(fig)

def ts_movie(data,tmin=None,tmax=None,fname='time_series',zone='Local',tunit='secs'):
    """
    Create time series movie

    Parameters
    ----------
    data : gwpy.timeseries.TimeSeries
      Time series data
    tmin : datetime
      First timestamp
    tmax : datetime
      Last timestamp
    fname : str
      Filename
    zone : str
      Output time zone, either UTC or Local
    tunit : str
      Scale of time axis (hrs, mins, secs)
    """
    t0,t1 = data.times.value[0],data.times.value[-1]
    tmin,tmax = (t0,t1) if tmin==tmax==None else str2timestamp(tmin,tmax)
    for time in numpy.arange(tmin,tmax,600):
        t0 = datetime.utcfromtimestamp(time)
        t1 = datetime.utcfromtimestamp(time+600)
        plot_time_series(data,t0,t1,fname='video_%s'%time,zone='Local',tunit='secs')
    os.system('convert -delay 60 -loop 0 -dispose previous video_*.png movie.gif')
    os.system('rm video_*.png')

def plot_wavelet(data,tmin=None,tmax=None,fmin=None,fmax=None,vmin=None,vmax=None,omega0=6,dt=1,dj=0.05,
                 fct='morlet',funit='Hz',cmap='inferno',zone='Local',tunit='secs',fname='wavelet',tbs=False):
    """
    Create wavelet spectrogram

    Parameters
    ----------
    data : gwpy.timeseries.TimeSeries
      Time series data
    tmin : datetime
      First timestamp
    tmax : datetime
      Last timestamp
    omega0 : int
      Wavelet function parameter
    dt : float
      Time step
    dj : float
      Scale resolution (smaller values of dj give finer resolution)
    fct : str
      Wavelet function (morlet,paul,dog)
    tbs : bool
      To Be Saved flag
    """
    # Define timestamps if not defined by user
    tmin = data.times[0].value  if tmin==None else tmin
    tmax = data.times[-1].value if tmax==None else tmax
    # Re-determine the mask in case timestamps are defined
    mask = (data.times.value>=tmin-1) & (data.times.value<=tmax+1)
    # Calculate wavelet parameters
    scales = mlpy.wavelet.autoscales(N=len(data[mask].value),dt=dt,dj=dj,wf=fct,p=omega0)
    spec = mlpy.wavelet.cwt(data[mask].value,dt=dt,scales=scales,wf=fct,p=omega0)
    freq = (omega0 + numpy.sqrt(2.0 + omega0 ** 2)) / (4 * numpy.pi * scales[1:])
    spec = numpy.abs(spec)**2
    spec = spec[::-1]
    # Convert time array into minute unit
    scale_factor = 3600. if tunit=='hrs' else 60. if tunit=='mins' else 1
    times = (data[mask].times.value-tmin)/scale_factor
    # Define minimum and maximum frequencies
    freq = freq * 1000. if funit=='mHz' else freq
    # Determine
    fmin_log,fmax_log = min(freq),max(freq)
    fmin_linear,fmax_linear = min(freq),max(freq)
    if fmin!=None:
        log_ratio = (numpy.log10(fmin)-numpy.log10(min(freq)))/(numpy.log10(max(freq))-numpy.log10(min(freq)))
        fmin_linear = min(freq)+log_ratio*(max(freq)-min(freq))
        fmin_log = fmin
    if fmax!=None:
        log_ratio = (numpy.log10(fmax)-numpy.log10(min(freq)))/(numpy.log10(max(freq))-numpy.log10(min(freq)))
        fmax_linear = min(freq)+log_ratio*(max(freq)-min(freq))
        fmax_log = fmax
    # Get minimum and maximum amplitude in selected frequency range
    idx = numpy.where(numpy.logical_and(fmin_log<freq[::-1],freq[::-1]<fmax_log))[0]
    vmin = spec[idx].min() if vmin==None else vmin
    vmax = spec[idx].max() if vmax==None else vmax
    # Initialise figure
    fig,ax = plt.subplots(figsize=(12,7))
    # Plot spectrogram
    img = ax.imshow(spec,extent=[times[0],times[-1],freq[-1],freq[0]],aspect='auto',
                    interpolation='nearest',cmap=cmap,norm=matplotlib.colors.LogNorm(vmin,vmax))
    ax.set_xlabel('Time [%s] from %s %s (%s)'%(tunit,datetime.utcfromtimestamp(tmin),zone,tmin),fontsize=15)
    ax.set_xlim(0,(tmax-tmin)/scale_factor)
    ax.set_yscale('linear')
    ax.set_ylim(fmin_linear,fmax_linear)
    ax.set_ylabel('Frequency [%s]'%funit,fontsize=15,labelpad=40)
    ax.grid(False)
    # Set up axis range for spectrogram
    twin_ax = ax.twinx()
    twin_ax.set_yscale('log')
    twin_ax.set_xlim(0,(tmax-tmin)/scale_factor)
    twin_ax.set_ylim(fmin_log,fmax_log)
    twin_ax.spines['top'].set_visible(False)
    twin_ax.spines['right'].set_visible(False)
    twin_ax.spines['bottom'].set_visible(False)
    ax.tick_params(which='both', labelleft=False, left=False)
    twin_ax.yaxis.tick_left()
    twin_ax.grid(False)
    # Add color bar and save figure
    cb = fig.colorbar(img,ax=ax,pad=0.01)
    cb.set_ticks(LogLocator())
    cb.set_label('Power $|\mathrm{W}_v|^2$ $[\mu T^2/\mathrm{Hz}]$',fontsize=15)
    cb.set_clim(vmin,vmax)
    plt.tight_layout()
    plt.savefig('%s.png'%fname) if tbs else plt.show()
    plt.close(fig)

def plot_psd(data,tmin=None,tmax=None,fname='psd',tbs=False):
    """
    Plot Power Spectral Density

    Parameters
    ----------
    data : gwpy.timeseries.TimeSeries
      Time series data
    tmin : datetime
      First timestamp
    tmax : datetime
      Last timestamp
    """
    # Define timestamps if not defined by user
    tmin = data.times[0].value  if tmin==None else tmin
    tmax = data.times[-1].value if tmax==None else tmax
    # Re-determine the mask in case timestamps are defined
    mask = (data.times.value>=tmin) & (data.times.value<=tmax)
    # Calculate PSD
    N = len(data[mask].value)
    dt = 1/data.sample_rate.value
    df = 1. / (N * dt)
    f = df * numpy.arange(N / 2)
    PSD = abs(dt * fftpack.fft(data[mask].value)[:N / 2]) ** 2
    psd = numpy.vstack((f,PSD)).T
    # Do the plotting
    fig = plt.figure(figsize=(12,7))
    plt.loglog(psd[:,0],psd[:,1])
    plt.xlabel('Frequency [Hz]')
    plt.ylabel('Power Spectral Density [Hz$^{-1}$]')
    plt.savefig('%s.png'%fname) if tbs else plt.show()
    plt.close(fig)

def plot_specgram(data,tmin=None,tmax=None,fmin=None,fmax=None,vmin=None,vmax=None,tbs=False,
                  funit='Hz',cmap='viridis',zone='Local',tunit='secs',fname='spectrogram',
                  stride=None,nfft=None,overlap=None):
    """
    Plot Fourier spectrogram.

    Parameters
    ----------
    data : gwpy.timeseries.TimeSeries
      Time series data
    tmin : datetime
      First timestamp
    tmax : datetime
      Last timestamp
    tbs : bool
      To Be Saved flag
    """
    # Define timestamps if not defined by user
    tmin = data.times[0].value  if tmin==None else tmin
    tmax = data.times[-1].value if tmax==None else tmax
    # Re-determine the mask in case timestamps are defined
    mask = (data.times.value>=tmin-1) & (data.times.value<=tmax+1)
    freq, times, spec = signal.spectrogram(data[mask],fs=data.sample_rate.value,
                                           nperseg=stride,noverlap=overlap,nfft=nfft)
    times = (data[mask].times.value[::int(stride-overlap)]-tmin)/scale_factor
    # Convert time array into minute unit
    scale_factor = 3600. if tunit=='hrs' else 60. if tunit=='mins' else 1
    #times = (data[mask].times.value-tmin)/scale_factor
    # Define minimum and maximum frequencies
    freq = freq * 1000. if funit=='mHz' else freq
    fmin = freq[1]      if fmin==None    else fmin
    fmax = max(freq)    if fmax==None    else fmax
    # Get minimum and maximum amplitude in selected frequency range
    idx = numpy.where(numpy.logical_and(fmin<=freq,freq<=fmax))[0]
    vmin = vmin if vmin!=None else numpy.sort(numpy.unique(spec[idx]))[1]
    vmax = spec[idx].max() if vmax==None else vmax
    # Initialise figure
    fig,ax = plt.subplots(figsize=(12,7))
    # Plot spectrogram
    img = ax.pcolormesh(times,freq,spec,cmap='inferno',norm=matplotlib.colors.LogNorm())
    ax.set_xlabel('Time [%s] from %s %s (%s)'%(tunit,datetime.utcfromtimestamp(tmin),zone,tmin),fontsize=15)
    ax.set_xlim(0,(tmax-tmin)/scale_factor)
    ax.set_ylim(fmin,fmax)
    ax.set_yscale('log')
    ax.set_ylabel('Frequency [%s]'%funit,fontsize=15,labelpad=40)
    ax.grid(False)
    # Add color bar and save figure
    cb = fig.colorbar(img,ax=ax,pad=0.01)
    cb.set_ticks(LogLocator())
    cb.set_label('Power $|\mathrm{W}_v|^2$ $[\mu T^2/\mathrm{Hz}]$',fontsize=15)
    cb.set_clim(vmin,vmax)
    plt.tight_layout()
    plt.savefig('%s.png'%fname) if tbs else plt.show()
    plt.close(fig)

def plot_availability(path,period,sensor,station):
    """
    Plot data availability.

    Examples
    --------
    >>> nuri availability --path ~/Cloud/Google/MagneticFieldData/ \\
    >>>                   --period 2016-5-2 2016-6-6 --station 4
    """
    if period==None or station==None:
        print('Period or station not defined. Abort.')
        quit()
    # Convert input dates to readable format
    tmin = nuri.str2datetime(period[0])
    tmax = nuri.str2datetime(period[1])
    # Determine timing offset based on original file construction
    offset = data_offset(tmin,tmax,station) if sensor=='biomed' else 0
    # Loop through all requested hours
    i,data_list = 0,numpy.empty((0,2))
    for date in numpy.arange(tmin+timedelta(seconds=offset),tmax+timedelta(seconds=offset),timedelta(hours=1)):
        # Convert date into readable datetime format
        date = date.astype(datetime)
        # Determine path to compressed timing binary file path from Google structure
        data_path = '%s/%s/%s/%s/%s/NURI-station-%02i/'%(path,date.year,date.month,date.day,date.hour,station)+date.strftime("%Y-%-m-%-d_%-H-xx.zip")
        data_flag = 0 if len(glob.glob(data_path))==0 else 1
        # Determine customized path to timing binary file
        data_path = '%s/NURI-station-%02i/'%(path,station)+date.strftime("%Y-%-m-%-d_%-H-xx_time.bin")
        data_flag = data_flag if len(glob.glob(data_path))==0 else 1
        # Determine customized path to timing binary file
        data_path = '%s/NURI-station-%02i/'%(path,station)+date.strftime("%Y-%m-%d-%H_*.bin")
        data_flag = data_flag if len(glob.glob(data_path))==0 else 1
        data_list = numpy.vstack((data_list,[date,data_flag]))
        if data_flag==0:
            print('Data file not found for %s'%date.strftime("%Y-%m-%d-%H"))
        i+=1
    # Create figure
    fig = plt.figure(figsize=(12,5))
    plt.style.use('seaborn')
    plt.rc('font', size=14, family='sans-serif')
    plt.rc('axes', labelsize=20, linewidth=0.2)
    plt.rc('legend', fontsize=12, handlelength=10)
    plt.rc('xtick', labelsize=12)
    plt.rc('ytick', labelsize=12)
    plt.scatter(data_list[:,0],data_list[:,1])
    plt.gca().xaxis.set_major_formatter(mdates.DateFormatter('%m/%d'))
    plt.gcf().autofmt_xdate()
    plt.xlim(data_list[0,0],data_list[-1,0])
    fig.tight_layout()
    plt.show()
    quit()



###############################################################################
# Copyright (c) 2018, Urban Magnetometry Group.
#
# Produced at the University of California at Berkeley.
#
# Written by V. Dumont (vincentdumont11@gmail.com).
#
# All rights reserved.
#
# This file is part of NURI.
#
# For details, see github.com/vincentdumont/nuri.
#
# For details about use and distribution, please read NURI/LICENSE.
###############################################################################
import nuri,os,numpy
from datetime import datetime,timedelta

def make_hour_spec(t0,t1,station=None):
    """
    Download data for specific period. The default period downloaded will
    be the last 24 hours.

    Parameters
    ----------
    station : float
      Station number. Default is all stations.
    start : str
      Starting date of the data to be retrieved, YYYY-MM-DD-HH
    end : Ending date of the data to be retrieved, YYYY-MM-DD-HH

    Examples
    --------
    >>> nuri_download --start 2016-03-20 --end 2016-03-21 --station 4
    """
    # Convert start and end date into datetime object
    d0 = datetime(*numpy.array(t0.split('-'),dtype=int)) if t0!=None else datetime(2015,11,1)
    d1 = datetime(*numpy.array(t1.split('-'),dtype=int)) if t1!=None else datetime.now()+timedelta(days=1)
    # Store data for current month
    os.system('skicka ls -r /MagneticFieldData/ > data')
    data = numpy.loadtxt('data',dtype=str,delimiter='\n')
    # Loop through every hour
    for d in numpy.arange(d0,d1,timedelta(hours=1)):
        # Extract individual parameters from datetime object
        year  = d.astype(object).year
        month = d.astype(object).month
        day   = d.astype(object).day
        hour  = d.astype(object).hour
        # Define remote path to and filename of the compressed archive
        path  = 'MagneticFieldData/%i/%i/%i/%i/'%(year,month,day,hour)
        fzip  = '%i-%i-%i_%i-xx.zip'%(year,month,day,hour)
        # Loop through each station
        for i in range(4):
            # Define full path to archive
            fullpath = path+'NURI-station-%02i/'%(i+1)+fzip
            # Check if station is targeted and remote path exists
            if (station==None or int(station)==i+1) and fullpath in data:
                # Download, uncompress and store data
                os.system('mkdir -p ./NURI-station-%02i/'%(i+1))
                os.system('skicka download /%sNURI-station-%02i/%s ./'%(path,i+1,fzip))
                os.system('unzip '+fzip)
                os.system('mv %i-%i-%i_%i-xx_* NURI-station-%02i'%(year,month,day,hour,i+1))
                t0 = '%i-%i-%i-%i'%(year,month,day,hour)
                dnext = d.astype(object)+timedelta(hours=1)
                t1 = '%i-%i-%i-%i'%(dnext.year,dnext.month,dnext.day,dnext.hour)
                mag_data = get_data(t0,t1,downfactor=3960,station=i+1,rep='./')
                tmin,tmax = str2timestamp(d.astype(object),dnext)
                fname = 'NURI-station-%02i/%i-%02i-%02i_%02i'%(i+1,year,month,day,hour)
                plot_spectrogram(mag_data,tmin,tmax,fscale='mHz',scale='mins',tbs=True,fname=fname)
                os.system('rm %s NURI-station-%02i/%i-%i-%i_%i-xx_*'%(fzip,i+1,year,month,day,hour))
    os.system('rm data')



###############################################################################
# Copyright (c) 2018, Urban Magnetometry Group.
#
# Produced at the University of California at Berkeley.
#
# Written by V. Dumont (vincentdumont11@gmail.com).
#
# All rights reserved.
#
# This file is part of NURI.
#
# For details, see github.com/vincentdumont/nuri.
#
# For details about use and distribution, please read NURI/LICENSE.
###############################################################################

import sys,numpy,time
from datetime import datetime
from gwpy.timeseries import TimeSeries

def is_float(a):
    try:
        float(a)
        return True
    except ValueError:
        return False

def biomed_read():

    setup.down    = 1
    setup.rescale = True
    #setup.sample  = False
    #setup.start   = '2016-9-15-6'
    #setup.end     = '2016-9-15-7'
    # Get start and ending timestamps
    start,tmin,end,tmax = time_edge(setup.start,setup.end)
    # Get NURI data
    path  = '/Users/vincent/ASTRO/data/DAQ_NURI1/'
    data  = magfield(setup.start,setup.end,rep=path)
    quit()
    # Create x axis time label
    tbeg = datetime.fromtimestamp(tmin)
    tend = datetime.fromtimestamp(tmax)
    sbeg = tbeg.strftime('%Y-%m-%d %H:%M:%S.%f')
    send = tend.strftime('%Y-%m-%d %H:%M:%S.%f')
    label1 = 'UTC time from %s to %s [secs]'%(sbeg,send)
    label2 = '%s-%s.png'%(int(tmin),int(tmax))
    # Plot pulse delay only
    fig = figure(figsize=(10,6))
    plt.subplots_adjust(left=0.1, right=0.95,
                        bottom=0.1, top=0.95,
                        hspace=0.1, wspace=0)
    ax = plt.subplot(111)
    print(min(data[:,1]),max(data[:,1]))
    ax.plot(data[:,0],data[:,1])
    ax.set_xlabel(label1)
    savefig(label2)
    clf()

def make_video():
    
    os.system('ls *.png > list.dat')
    filelist = sorted(numpy.loadtxt('list.dat',dtype=str))
    for i in range(len(filelist)):
        newpath = 'series%04i.png'%(i+1)
        if os.path.exists(newpath)==False:
            os.system('cp %s %s'%(filelist[i],newpath))
    os.system('ffmpeg -framerate 30 -i series%04d.png  -start_number 1 -c:v libx264 -r 30 -pix_fmt yuv420p ../video.mp4')
    os.system('rm list.dat series*')
    
def str2timestamp(*times):
    """
    Create mask for user-defined time range.
    """
    time_list = []
    for dt in times:
        # Split date by dashes
        dt = numpy.array(dt.split('-'),dtype=int)
        # Include day 1 if no day provided
        dt = dt if len(dt)>2 else numpy.hstack((dt,1))
        # Convert string to datetime format
        dt = datetime(*dt)
        # Convert start date into timestamp
        dt = time.mktime(dt.timetuple())+dt.microsecond/1e6
        # Convert timestamp back local datetime
        now = datetime.fromtimestamp(dt)
        # Convert timestamp to UTC datetime
        utc_now = datetime.utcfromtimestamp(dt)
        # Calculate local difference with UTC time
        utc2local = (now-utc_now).total_seconds()
        # Remove UTC offset from timestamp
        dt += utc2local
        time_list.append(dt)
    return tuple(time_list) if len(time_list)>1 else time_list[0]

def str2datetime(*dates):
    '''
    Convert input string date to datetime format.
    '''
    date_list = []
    for date in dates:
        dstr = ['%Y','%m','%d','%H','%M','%S','%f']
        dsplit = '-'.join(dstr[:date.count('-')+1])
        date = datetime.strptime(date,dsplit)
        date_list.append(date)
    return tuple(date_list) if len(date_list)>1 else date_list[0]

def get_scalar(data):
    '''
    Calculate scalar signal from each directional magnetic field data.

    Parameters
    ----------
    data : numpy.array
      Timestamps and directional magnetic field signals
    
    Return
    ------
    data : numpy.array
      Return modified data array with scalar values
    '''
    scalar = numpy.sqrt(numpy.sum(abs(data[:,1:])**2,axis=1,dtype=float))
    return numpy.vstack((data[:,0],scalar)).T




###############################################################################
# Copyright (c) 2018, Urban Magnetometry Group.
#
# Produced at the University of California at Berkeley.
#
# Written by V. Dumont (vincentdumont11@gmail.com).
#
# All rights reserved.
#
# This file is part of NURI.
#
# For details, see github.com/vincentdumont/nuri.
#
# For details about use and distribution, please read NURI/LICENSE.
###############################################################################
class fftcomp:
    
    """
    This operation will plot the Fourier transform of given time series.
    The time series and FFT are shown interactively so the user can select
    a desired time slot and get the FFT from that region.

    Parameters
    ----------
    start : str
      Starting date to retrieve data from.
    end : str 
      Ending date to retrieve data from.
    station : str
      Station name
    discard : float
      Discard first selected seconds
    fmin : 1
      Minimum frequency to display
    fmax : float
      Maximum frequency to display
    nogps : bool 
      No GPS provided, create time array using sample rate
    sample : bool
      Specify the data to be field sample, not regular.
    scale : float
      Rescale y axis
    unit : str
      Time unit for plot.
    
    Examples
    --------
    >>> nuri fftcomp --start 2017-03-09-15-58 --end 2017-03-10 \
    >>>              --path /Users/vincent/ASTRO/data/others/sample1_background_noise/ \
    >>>              --start2 2017-03-09-16-2 --end2 2017-03-09-16-4 \
    >>>              --path2 /Users/vincent/ASTRO/data/others/sample2_magnet_test1/ \
    >>>              --station magnet --sample --unit secs --nogps \
    >>>              --discard 1 --fmin 1 --down 12 --seaborn
    """

    def __init__(self,start,end,station,discard=0,fmin=1,fmax=None,nogps=False,sample=False,scale=1,unit='hrs'):
        
        if setup.station==None or setup.start==None or setup.end==None:
            print('\nERROR: station or start or end dates not specified...\n')
            quit()
            
        t1,x1,y1,z1,f1,fft_x1,fft_y1,fft_z1 = self.get_data(setup.start,setup.end,setup.path)
        t2,x2,y2,z2,f2,fft_x2,fft_y2,fft_z2 = self.get_data(setup.start2,setup.end2,setup.path2)
        self.plot_data(t1,x1,t2,x2,f1,fft_x1,f2,fft_x2,'X')
        self.plot_data(t1,y1,t2,y2,f1,fft_y1,f2,fft_y2,'Y')
        self.plot_data(t1,z1,t2,z2,f1,fft_z1,f2,fft_z2,'Z')

    def plot_data(self,t1,y1,t2,y2,f1,fft_y1,f2,fft_y2,direction):
        """
        Plot data
        """
        # Initialise axis
        fig = figure(figsize=(15,10))
        plt.subplots_adjust(left=0.07,right=0.95,bottom=0.1,top=0.95,hspace=0.2,wspace=0.05)
        # Plot time series
        ax1 = subplot(221)
        ax1.plot(t1,y1)
        ax1.set_xlabel('Time [%s]'%setup.unit)
        ax1.set_ylabel('Magnetic field [uT]')
        ax1.set_title('Without Magnet - %s direction'%direction)
        # Plot time series
        ax2 = subplot(222,sharex=ax1,sharey=ax1)
        ax2.plot(t2,y2)
        ax2.set_xlim(t2[0],t2[-1])
        ax2.set_xlabel('Time [%s]'%setup.unit)
        ax2.set_title('With Magnet - %s direction'%direction)
        plt.setp(ax2.get_yticklabels(),visible=False)
        # Plot FFT for x direction
        ax3 = subplot(223)
        ax3.semilogy(f1,fft_y1)
        ax3.set_xlabel('Frequency [Hz]')
        ax3.set_ylabel('|Y(f)|')
        # Plot FFT for y direction
        ax4 = subplot(224,sharex=ax3,sharey=ax3)
        ax4.semilogy(f2,fft_y2)
        ax4.set_xlim(1,f2[-1])
        ax4.set_xlabel('Frequency [Hz]')
        plt.setp(ax4.get_yticklabels(),visible=False)
        # Save figure
        plt.savefig('%s-fft-%s.png'%(setup.station,direction))
        plt.close(fig)

    def get_data(self,start,end,path):
    
        t,x,y,z = magfield(start,end,rep=path)
        # Discard selected data
        t = t[setup.rate*setup.discard:]
        x = x[setup.rate*setup.discard:]
        y = y[setup.rate*setup.discard:]
        z = z[setup.rate*setup.discard:]
        # Decimate magnetic field data to 1 sample/second
        down_factor = [setup.down] if setup.down<13 else \
                      [11,9,4] if setup.down==396 else [11,10,9,4]
        for i in down_factor:
            x = signal.decimate(x,i,zero_phase=True)
            y = signal.decimate(y,i,zero_phase=True)
            z = signal.decimate(z,i,zero_phase=True)
        # Extract time every 3960 sample
        l = len(t)/setup.down
        l = l if l*setup.down==len(t) else l+1
        t = [t[n*setup.down] for n in range(l)]
        if setup.nogps: t = [float(n)*setup.down/setup.rate for n in range(len(x))]
        # Convert every timing points to scale (hr,min,sec) units
        s = 1. if setup.unit=='secs' else 60. if setup.unit=='mins' else 3600.
        t = [(t[i]-t[0])/s for i in range(len(t))]
        # Create FFT of data
        N = len(t)
        T = 1.0 / (setup.rate/setup.down)
        f = np.linspace(0.0, 1.0/(2.0*T), N/2)
        i1 = abs(f-setup.fmin).argmin()
        i2 = len(f) if setup.fmax==None else abs(f-setup.fmax).argmin()
        f = f[i1:i2]
        fft_x = 2.0/N * np.abs(fft(x)[:N//2])[i1:i2]
        fft_y = 2.0/N * np.abs(fft(y)[:N//2])[i1:i2]
        fft_z = 2.0/N * np.abs(fft(z)[:N//2])[i1:i2]
        return t,x,y,z,f,fft_x,fft_y,fft_z

if __name__=='__main__':
    argp = argparse.ArgumentParser()
    argp.add_argument("--date", help="Set date")
    args = argp.parse_args()
    active(args.date)



###############################################################################
# Copyright (c) 2018, Urban Magnetometry Group.
# Produced at the University of California at Berkeley.
# Written by V. Dumont (vincentdumont11@gmail.com).
# All rights reserved.
# This file is part of NURI.
# For details, see http://citymag.gitlab.io/nuri
# For details about use and distribution, please read NURI/LICENSE.
###############################################################################
import time,numpy,glob,numpy,re,struct,astropy,h5py,sys,math,os,pytz,zipfile
from astropy.time    import Time
from datetime        import datetime,timedelta,date
from dateutil        import tz
from gwpy.timeseries import TimeSeries
from itertools       import combinations
from scipy           import signal
from .utils          import str2datetime,str2timestamp

def get_data(tmin=None,tmax=None,path=None,sensor=None,station=None,rate=None):
    """
    Extract time-series data from specific path.
    """
    # Create dataset
    ext = path.split('.')[-1]
    if ext=='bin':
        dataset = [path]
        offset = 0
    else:
        # Convert input dates to readable format
        tmin = str2datetime(tmin)
        tmax = str2datetime(tmax)
        # Determine timing offset based on original file construction
        offset = data_offset(tmin,tmax,station) if sensor=='biomed' else 0
        dataset = data_lookup(path.replace('\ ',' '),tmin,tmax,station,offset)
    # Retrieve the magnetic field data
    assert len(dataset)>0, 'No data file found. Abort.'
    if sensor=='twinleaf':
        data = get_vmr_data(dataset,rate)
    elif '-xx' in dataset[0]:
        data = data_extractor(dataset,offset,rate,sensor)
    else:
        data = data_extractor(dataset,offset,rate)
    return data

def data_lookup(path,tmin,tmax,station=None,offset=0):
    """
    Lookup any Biomed data files within user-defined repository path. If direct path
    to binary file provided, this will simply return a list with the given path as a
    single element. Alternatively, and if a specific time period is provided by the
    user, this function will look for either zip files from the Google drive directory
    structure (i.e. year/month/day/hour/station) or the customised architecture
    (i.e. station/).
    
    Parameters
    ----------
    path : `str`
      Path to data file or repository
    tmin : str
      Starting time in format YYYY-MM-DD-HH-MM
    tmax : str
      Ending time in format YYYY-MM-DD-HH-MM
    station : int
      Station number

    Returns
    -------
    dataset : list
      List of paths to identified data files
    """
    if station==None or tmin==None or tmax==None:
        print('Station, tmin and tmax must be defined. Abort.')
        quit()
    # Initialise dataset arrays
    zip_list,biomed_list,data_list = [],[],[]
    # Loop through all requested hours
    for date in numpy.arange(tmin+timedelta(seconds=offset),tmax+timedelta(seconds=offset),timedelta(hours=1)):
        # Convert date into readable datetime format
        date = date.astype(datetime)
        # Determine path to compressed timing binary file path from Google structure
        zip_path = '%s/%s/%s/%s/%s/NURI-station-%02i/'%(path,date.year,date.month,date.day,date.hour,station)+date.strftime("%Y-%-m-%-d_%-H-xx.zip")
        zip_list = check_continuity(zip_path,zip_list)
        # Determine customized path to timing binary file
        biomed_path = '%s/NURI-station-%02i/'%(path,station)+date.strftime("%Y-%-m-%-d_%-H-xx_time.bin")
        biomed_list = check_continuity(biomed_path,biomed_list)
        # Determine customized path to timing binary file
        data_path = '%s/NURI-station-%02i/'%(path,station)+date.strftime("%Y-%m-%d-%H_*.bin")
        data_list = check_continuity(data_path,data_list)
    # Select dataset
    dataset = zip_list if len(zip_list)>0 else biomed_list if len(biomed_list)>0 else data_list
    return dataset

def check_continuity(path,dataset):
    """
    If dataset already non-empty, check that subsequent data file is found.

    Parameters
    ----------
    path : `str`
      Assumed path to next data file, the algorithm will check if file exists
    dataset : `str`
      Most recent list of data file actually found so far and up to current searched date
    """
    assert len(dataset)==0 or (len(dataset)>0 and len(glob.glob(path))>0), \
        "No file found for date %s"%re.split('\/|_',path)[-2]
    dataset += glob.glob(path)
    return dataset

def data_offset(tmin,tmax,station,add=False):
    """
    Determine UTC to local Local offset to be applied.
    
    Parameters
    ----------
    tmin : datetime
      Starting time
    tmax : str
      Ending time
    station : int
      Station number
    output : str
      Offset value requested

    Retu>rn
    ------
    offset : float
      Offset time to match time in targeted filename
    utc2pst : float
      Time offset from UTC to PST time
    """
    # Identifying the time zone
    utc_zone = tz.gettz('UTC')
    # Format input timestamp into UTC time
    utc_epoch = tmin.replace(tzinfo=utc_zone)
    # Get time in local California time
    local_epoch = utc_epoch.astimezone(tz.gettz('America/Los_Angeles'))
    # Calculate offset between UTC and PST timestamps
    utc2pst = datetime.utcoffset(local_epoch).total_seconds()
    # Consider UTC to PST offset if requested time is before fix date
    utc2pst = utc2pst if tmin<datetime(2017,12,7) else 0
    # Look-up table to identify station's location over time
    locations = numpy.array([[1,datetime(2015,11,1),datetime(2017,12,3),tz.gettz('America/Los_Angeles')],
                             [1,datetime(2017,12,3),datetime.max       ,tz.gettz('America/New_York')   ],
                             [2,datetime(2015,11,1),datetime(2018,6,7) ,tz.gettz('America/Los_Angeles')],
                             [2,datetime(2018,6,7) ,datetime.max       ,tz.gettz('America/New_York'   )],
                             [3,datetime(2015,11,1),datetime(2017,10,6),tz.gettz('America/Los_Angeles')],
                             [3,datetime(2017,10,6),datetime.max       ,tz.gettz('America/New_York')   ],
                             [4,datetime(2015,11,1),datetime(2017,12,3),tz.gettz('America/Los_Angeles')],
                             [4,datetime(2017,12,3),datetime.max       ,tz.gettz('America/New_York')   ]])
    # Identify the location for requested data
    for n,start,end,loc in locations:
        if n==station and start<tmin<end:
            local_zone = loc
    # Identifying the time zone
    utc_zone = tz.gettz('UTC')
    # Format input timestamp into UTC time
    utc_epoch = tmin.replace(tzinfo=utc_zone)
    # Get time in local California time
    local_epoch = utc_epoch.astimezone(local_zone)
    # Calculate offset between Local and UTC timestamps
    utc2local = datetime.utcoffset(local_epoch).total_seconds()
    # Define offset to local time for new timing files
    offset = 0 if tmax<datetime(2016,6,10) else -utc2local
    return utc2pst-offset if add else offset

def data_extractor(dataset,offset=0,rate=None,sensor=None,scalar=True):
    """
    Retrieve data time series

    Parameters
    ----------
    rate : int
      Targeted sampling rate
    scalar : bool
      Type of output value, default is the scalar magnitude among all directions.
    
    Returns
    -------
    ts : gwpy.timeseries.TimeSeries
      Time series data, either scalar magnitude or for individual direction.
    """
    data_list = []
    if sensor=='biomed':
        t0 = numpy.array(re.split('\/|-xx',dataset[0]))[-2].replace('_','-')
        t1 = numpy.array(re.split('\/|-xx',dataset[-1]))[-2].replace('_','-')
        for filename in dataset:
            # Extract data from X direction file
            xfile = filename.split('-xx')[0]+'-xx_rawX_uT_3960Hz.bin'
            x = read_axis(xfile,rate)
            # Extract data from Y direction file
            yfile = filename.split('-xx')[0]+'-xx_rawY_uT_3960Hz.bin'
            y = read_axis(yfile,rate)
            # Extract data from Z direction file
            zfile = filename.split('-xx')[0]+'-xx_rawZ_uT_3960Hz.bin'
            z = read_axis(zfile,rate)
            # Define 2D array
            ts_data = numpy.vstack((x,y,z)).T
            data_list.append(ts_data)
    else:
        t0 = numpy.array(re.split('\/|_',dataset[0]))[-2]
        t1 = numpy.array(re.split('\/|_',dataset[-1]))[-2]
        for filename in dataset:
            # Read binary file and sore data in array
            with open(filename,'rb') as f:
                ts_data = f.read()
            f.close()
            # Define the total number records (24 bytes per record)
            size = int(len(ts_data)/24)
            # Unpack each record as the following succession:
            ts_data = struct.unpack('ddd'*size,ts_data)
            # Reshape array into 2D format
            ts_data = numpy.array(ts_data,dtype=float).reshape((size,3))
            data_list.append(ts_data)
    # Concatenate all time series in list
    data = numpy.concatenate(data_list)
    # Create timing array with equally separated timing sample
    t0,t1 = str2timestamp(t0,t1)
    times = numpy.linspace(t0-offset,t1-offset+3600,len(data),endpoint=False)
    rate  = 1./((t1+3600-t0)/len(data))
    # Create output data array
    if scalar:
        data = numpy.sqrt(numpy.sum(abs(data[:,1:])**2,axis=1))
        data = TimeSeries(data,sample_rate=rate,epoch=t0-offset)
    else:
        x = TimeSeries(data[:,0],sample_rate=rate,epoch=t0)
        y = TimeSeries(data[:,1],sample_rate=rate,epoch=t0)
        z = TimeSeries(data[:,2],sample_rate=rate,epoch=t0)
        data = (x,y,z)
    return data

def biomed_converter(dataset,tmin,tmax,station,rate=3960,dest=None):
    """
    Convert binary data into new binary files. This function was optimized to prepare the
    dataset for an entire month of data. Several downsample version of the data will be
    produced: 3960Hz, 1980Hz, 990Hz, 198Hz, 1Hz, and 1S/min.

    Examples
    --------
    >>> nuri convert --path ~/Cloud/Google/MagneticFieldData/ \\ 
    >>>              --dest ~/Cloud/Dropbox/citymag/data/ --rate 1 \\
    >>>              --period 2016-5-15-18 2016-5-21 --station 4
    """
    # Define data path and move to repository
    data_path = os.getenv('HOME')+'/nuri_data' if dest==None else dest
    # Initialise arrays and loop over found datasets
    i=0
    for filename in dataset:
        # Go to station target folder
        os.system('mkdir -p %s/'%data_path)
        os.chdir('%s/'%data_path)
        # Check if target file is original Google Drive zip folder
        if '.zip' in filename and os.stat(filename).st_size>4e+7:
            print('> Extracting files from %s'%filename)
            tprocess = time.time()
            # Read zip file
            zip_rep = zipfile.ZipFile(filename, 'r')
            # Identify timing file in the compressed folder
            filename = filename.split('/')[-1].split('-xx')[-2]+'-xx_time.bin'
            filename = filename if filename in zip_rep.namelist() else filename.replace('.bin','_v2.bin')
            # Check if data already present in repository otherwise extract
            if os.path.exists(filename)==False:
                zip_rep.extractall()
            # Close the zip file reading
            zip_rep.close()
            print('  > Process time: %ss'%(time.time()-tprocess))
        if '.bin' in filename:
            print('  > Creating data with %s sampling rate'%rate)
            tprocess = time.time()
            # Extract data from X direction file
            xfile = filename.split('-xx')[0]+'-xx_rawX_uT_3960Hz.bin'
            x = read_axis(xfile,rate)
            # Extract data from Y direction file
            yfile = filename.split('-xx')[0]+'-xx_rawY_uT_3960Hz.bin'
            y = read_axis(yfile,rate)
            # Extract data from Z direction file
            zfile = filename.split('-xx')[0]+'-xx_rawZ_uT_3960Hz.bin'
            z = read_axis(zfile,rate)
            os.system('rm *xx*.bin')
            # If one array has lower length than others, discard first samples
            imax = min(len(x),len(y),len(z))
            # Create filename for converted file
            filename = numpy.array(re.split('-|_',filename)[:4],dtype=int)
            filename = '{0:>04}-{1:>02}-{2:>02}-{3:>02}'.format(*filename)+'_%ipts.bin'%imax
            # Save magnetic field data into new binary file
            magdata = (numpy.vstack([x[-imax:],y[-imax:],z[-imax:]]).T).ravel()
            # Move to destination path
            station_path = '/NURI-station-%02i/'%station
            os.system('mkdir -p %s/%iHz/%s/'%(data_path,rate,station_path))
            os.chdir('%s/%iHz/%s/'%(data_path,rate,station_path))
            # Create new binary file
            binfile = open(filename,'wb')
            binfile.write(struct.pack('d'*len(magdata), *magdata))
            binfile.close()
            print('  > Process time: %ss'%(time.time()-tprocess))
        
def biomed_timing(filename,rate,offset):
    """
    Read timing binary file and construct full timestamp array.

    Parameters
    ----------
    filename : str
      Path to the timing binary file
    offset : datetime
      Offset time to match time in targeted filename 
    
    Returns
    -------
    tgps : numpy.array
      Reconstructed full timing array
    """
    # Read binary file and sore data in array
    with open(filename,'rb') as f:
        data = f.read()
    f.close()
    # Check timing version
    if 'time_v2' in filename:
        # Define the total number records (63 bytes per record)
        size = len(data)/63
        # Unpack each record as the following succession:
        data = struct.unpack('<'+'qI?Qddcdcdd'*size,data)
        # Reshape array into 2D format
        data = numpy.array(data,dtype=object).reshape((size,11))
        # Fix chunk index column
        idxs = [i*138 for i in range(len(data))]
        # Create incremented sample index array
        x = numpy.array(idxs+data[:,1],dtype=float)
        # Defined timestamp array
        y = numpy.array(data[:,4]+offset)
        # Fit timestamp array
        popt,pcov = numpy.polyfit(x,y,1,cov=True)
        # Create full sample index array
        xfit = numpy.arange(x[-1])
        # Construct full timestamp array
        tgps = numpy.polyval(popt,xfit)
    else:
        # Define the total number records (28 bytes per record)
        size = len(data)/28
        # Unpack each record as the following succession:
        data = struct.unpack('<'+'qiQd'*size,data)
        # Reshape array so that each row corresponds to one record
        data = numpy.reshape(data,(size,4))
        # Convert timestamp into 
        data[:,2] = [(int(i) & 0x7fffffffffffffff)/1e7 for i in data[:,2]]
        # Correct timestamp from offset between system clock and UTC time
        data[:,2]-=(datetime(1970,1,1)-datetime(1,1,1)).total_seconds()
        # Create incremented sample index array
        x = data[:,0]+data[:,1]
        # Defined timestamp array
        y = data[:,2]+offset
        # Fit timestamp array
        popt,pcov = numpy.polyfit(x,y,1,cov=True)
        # Create full sample index array
        xfit = numpy.arange(x[-1])
        # Construct full timestamp array
        tgps = numpy.polyval(popt,xfit)
    # Define downsampling factor
    downfactor = 1 if rate==None else int(3960. / rate)
    # Check if sampling rate divisible by targeted sampling rate
    if rate!=None and 3960 % rate != 0:
        print('Targeted rate of %i is not a multiple of original sampling rate of %i. Abort.'%(rate,3960))
        quit()
    # Downsample time and data arrays
    tgps = tgps if downfactor==1 else tgps[::downfactor][:-1]
    return numpy.array(tgps)

def read_axis(filename,rate):
    """
    Read the binary file of magnetic field axis.

    Parameters
    ----------
    filename : str
      Path to data file

    Returns
    -------
    data : numpy.array
      Magnetic field data.
    """
    # Read binary file and sore data in array
    with open(filename,'rb') as f:
        data = f.read()
    f.close()
    # Define the total number records (63 bytes per record)
    size = len(data)/8
    # Unpack each record
    data = numpy.array(struct.unpack('d'*size,data))
    # Get median and standard deviation of time series
    med = numpy.median(data)
    std = numpy.std(data)
    # Look for outlier (10 times std) and fix value to median.
    idxs = numpy.where(abs(data-med)>10*std)[0]
    data[idxs] = med
    # Define downsampling factor
    downfactor = 1 if rate==None else int(3960. / rate)
    if downfactor>1:
        # Get closest multiple of the downsample factor from the data length
        limit = int(downfactor * round(float(len(data))/downfactor))
        # Ensure data length is multiple of downsampling factor
        data = data[:limit] if limit<=len(data) else data[:limit-downfactor]
        # Re-sample the data
        data = signal.resample(data,len(data)/downfactor)
    return data

def get_coord(filename):
    """
    Extracted longitude and latitude from version 2 timing data files.

    Parameters
    ----------
    filename : str
      Name of the timing binary file.

    Returns
    -------
    lat, lon : float
      Latitute and Longitude of the magnetic sensor.
    """
    with open(filename,'rb') as f:
        data = f.read()
    f.close()
    s = len(data)/63
    data = struct.unpack('<'+'qipQddcdcdd'*s,data)
    data = numpy.reshape(data,(s,11))
    lat  = float(str(float(data[0,5])/100).split('.')[0])
    lat  = lat + (float(data[0,5])-lat*100)/60
    lat  = -lat if data[0,8]=='S' else lat
    lon  = float(str(float(data[0,7])/100).split('.')[0])
    lon  = lon + (float(data[0,7])-lon*100)/60
    lon  = -lon if data[0,6]=='W' else lon
    return lat,lon

def get_gnome(station,t0,t1,local=False,
              rep='/Users/vincent/ASTRO/data/NURI/'):
    """
    Glob all files withing user-defined period and extract data.

    Parameters
    ----------
    station : str
      Name of the station to be analysed
    t0 : str
      Starting time in format YYYY-MM-DD-HH-MM
    t1 : str
      Ending time in format YYYY-MM-DD-HH-MM
    local : bool
      Whether we extract local (GPS) or universal (UTC) time.
    rep : str
      Path in which the data are stored.

    Returns
    -------
    data : numpy.array
      2D magnetic field data array.
    """
    print('Searching GNOME data files...')
    # Convert start and end dates into datetime objects
    t0 = datetime(*numpy.array(t0.split('-'),dtype=int))
    t1 = datetime(*numpy.array(t1.split('-'),dtype=int))
    dataset = []
    for date in numpy.arange(start,end,timedelta(minutes=1)):
        date = date.astype(datetime)
        fullpath = rep+'/'+station+'_'+date.strftime("%Y%m%d_%H%M*.h5")
        dataset += glob.glob(fullpath)
    if sample:
        dataset = glob.glob(rep+"/*.h5")
    gps_offset = (datetime(1980,1,6)-datetime(1970,1,1)).total_seconds()
    tdata,xdata = [],[]
    for fname in sorted(dataset):
        hfile = h5py.File(fname, "r")
        dset = hfile['MagneticFields']
        datestr = dset.attrs["Date"]
        t0str = dset.attrs["t0"]
        # Format date from extracted metadata
        instr = "%d-%d-%02dT" % tuple(map(int, datestr.split('/'))) + t0str
        # Calculate offset between UTC and GPS initial epoch time
        gps_offset = (datetime(1980,1,6)-datetime(1970,1,1)).total_seconds()
        # Calculate GPS epoch
        gps_epoch = astropy.time.Time(instr, format='isot', scale='utc').gps + gps_offset
        utc,gps     = datetime.utcfromtimestamp(gps_epoch),datetime.fromtimestamp(gps_epoch)
        delay       = round(abs((utc-gps).total_seconds())/3600.)
        start_time  = gps if local else utc
        start_time  = time.mktime(start_time.timetuple())
        sample_rate = dset.attrs["SamplingRate(Hz)"]
        end_time    = start_time + len(dset[:]) / sample_rate
        tarray      = numpy.linspace(start_time,end_time,len(dset[:]))
        t0 = time.mktime(t0.timetuple())+t0.microsecond/1e6
        t1 = time.mktime(t1.timetuple())+t1.microsecond/1e6
        idx = numpy.where(numpy.logical_and(t0-1<tarray,tarray<t1+1))[0]
        xdata       = numpy.hstack((xdata,dset[:][idx]))
        tdata       = numpy.hstack((tdata,tarray[idx]))
        hfile.close()
    data = numpy.vstack((tdata,10*xdata)).T
    return data

def get_vmr_data(path,rate=None,offset=0.,ufactor=1000,tmin=None,tmax=None,chunk_size=60):
    """
    Extract data from individual VMR data log file

    Parameters
    ----------
    sample_rate : int
      Actual sampling rate of the recorded data
    rate : int
      Targeted sampling rate after downsampling
    ufactor : float
      Magnetic field factor to convert into microTesla
    
    Returns
    -------
    data : gwpy.timeseries.TimeSeries
      Time series data of the scalar magnitude
    """
    # Read binary file and store floating values in numpy array
    with open(path,'rb') as f:
      data = f.read()
    f.close()
    size = int(len(data)/32)
    data = struct.unpack('dddd'*size,data)
    data = numpy.reshape(data,(size,4))
    data[:,0] += offset*3600.
    # Identify samples to discard
    if tmin!=None:
      idxs = numpy.where(data[:,0]<tmin)[0]
      data = numpy.delete(data,idxs,axis=0)
    if tmax!=None:
      idxs = numpy.where(data[:,0]>tmax)[0]
      data = numpy.delete(data,idxs,axis=0)
    # Derive data sampling rate from average dt between samples
    sample_rate = round(1./numpy.average([data[i+1,0]-data[i,0] for i in range(len(data)-1)]))
    target_rate = sample_rate if rate==None else rate
    downfactor = int(sample_rate / target_rate)
    if sample_rate % target_rate != 0:
        print('Targeted rate of %i is not a multiple of original sampling rate of %i. Abort.'%(target_rate,sample_rate))
        quit()
    # Calculate scalar value of magnetic field for each sample
    scalar = numpy.sqrt(numpy.sum(abs(data[:,1:]/ufactor)**2,axis=1))
    # Check if targeted sampling rate different than original rate
    if sample_rate != target_rate:
        # Get closest multiple of the downsample factor from the data length
        limit = int(downfactor * round(float(len(scalar))/downfactor))
        # Ensure data length is multiple of downsampling factor
        scalar = scalar[:limit] if limit<=len(scalar) else scalar[:limit-downfactor]
        # Re-sample the data
        scalar = signal.resample(scalar,len(scalar)/downfactor)
    # Print out first and last timestamp values
    date0 = datetime.fromtimestamp(data[0,0]).strftime('%Y-%m-%d %H:%M:%S.%f')
    date1 = datetime.fromtimestamp(data[-1,0]).strftime('%Y-%m-%d %H:%M:%S.%f')
    print('Data selected from %s to %s'%(date0,date1))
    # Store data into GwPy time series format
    data = TimeSeries(scalar,sample_rate=target_rate,epoch=data[0,0])
    return data
    
def downsample(data,downfactor):
    """
    Downsample data time series.
    """
    times = data[:,0] if downfactor==1 else data[::downfactor,0][:-1]
    # Get closest multiple of the downsample factor from the data length
    limit = int(downfactor * round(float(len(data))/downfactor))
    # Ensure data length is multiple of downsampling factor
    data = data[:limit,1] if limit<=len(data) else data[:limit-downfactor,1]
    # Re-sample the data
    data = signal.resample(data,len(data)/downfactor)
    # Create dataset
    data = numpy.vstack((times,data)).T
    return data



###############################################################################
# Copyright (c) 2018, Urban Magnetometry Group.
# Produced at the University of California at Berkeley.
# Written by V. Dumont (vincentdumont11@gmail.com).
# All rights reserved.
# This file is part of NURI.
# For details, see https://citymag.gitlab.io/nuri
# For details about use and distribution, please read NURI/LICENSE.
###############################################################################
import sys,os,numpy
import matplotlib.pyplot as plt
import matplotlib.dates as md
from datetime import datetime,timedelta

def check24hrs(date):
    """
    This operation will display the active periods for which data are
    available from every sensors.

    Parameters
    ----------

    date : str
      Year and month to display activity from. The format shoud be YYYY-MM.
    """
    # Check if date is given
    if date==None:
        print('date missing...')
        quit()
    # List all the months
    dates = numpy.empty((0,5))
    y0    = int(date.split('-')[0])
    m0    = int(date.split('-')[1])
    d0    = datetime(y0,m0,1)
    y1    = y0   if m0<12 else y0+1
    m1    = m0+1 if m0<12 else 1
    d1    = datetime(y1,m1,1)-timedelta(hours=1)
    dt    = timedelta(hours=1)
    dates = numpy.arange(d0,d1,dt)
    # Download metadata from Google Drive
    sys.stderr.write('Retrieve information from Google Drive...')
    os.system('skicka ls -r /MagneticFieldData/%s/%s/ > data'%(y0,m0))
    data = numpy.loadtxt('data',dtype=str,delimiter='\n')
    print(sys.stderr,' done!')
    # List file path for each date and each station
    sys.stderr.write('Select active hours for each station...')
    st0,st1,st2,st3,st4 = [],[],[],[],[]
    for d in dates:
        year  = d.astype(object).year
        month = d.astype(object).month
        day   = d.astype(object).day
        hour  = d.astype(object).hour
        path  = 'MagneticFieldData/%i/%i/%i/%i/'%(year,month,day,hour)
        fname = '%i-%i-%i_%i-xx.zip'%(year,month,day,hour)
        st0.append(path+'NURI-station/'   +fname)
        st1.append(path+'NURI-station-01/'+fname)
        st2.append(path+'NURI-station-02/'+fname)
        st3.append(path+'NURI-station-03/'+fname)
        st4.append(path+'NURI-station-04/'+fname)
    st0 = numpy.array([1 if path in data else 0 for path in st0])
    st1 = numpy.array([1 if path in data else 0 for path in st1])
    st2 = numpy.array([1 if path in data else 0 for path in st2])
    st3 = numpy.array([1 if path in data else 0 for path in st3])
    st4 = numpy.array([1 if path in data else 0 for path in st4])
    print(sys.stderr,' done!')
    # Write down information in text file
    print('Save information in ASCII file...')
    o = open('%i-%02i.dat'%(y0,m0),'w')
    for d in dates:
        year  = d.astype(object).year
        month = d.astype(object).month
        day   = d.astype(object).day
        hour  = d.astype(object).hour
        path  = 'MagneticFieldData/%i/%i/%i/%i/'%(year,month,day,hour)
        fname = '%i-%i-%i_%i-xx.zip'%(year,month,day,hour)
        o.write('%i-%02i-%02i_%02i'%(year,month,day,hour))
        o.write('  NURI-station   ') if path+'NURI-station/'   +fname in data else o.write('  -              ')
        o.write('  NURI-station-01') if path+'NURI-station-01/'+fname in data else o.write('  -              ')
        o.write('  NURI-station-02') if path+'NURI-station-01/'+fname in data else o.write('  -              ')
        o.write('  NURI-station-03') if path+'NURI-station-01/'+fname in data else o.write('  -              ')
        o.write('  NURI-station-04') if path+'NURI-station-01/'+fname in data else o.write('  -              ')
        o.write('\n')
    o.close()
    dates = [d.astype(object) for d in dates]
    plt.rc('font', size=2, family='serif')
    plt.rc('axes', labelsize=10, linewidth=0.2)
    plt.rc('legend', fontsize=2, handlelength=10)
    plt.rc('xtick', labelsize=7)
    plt.rc('ytick', labelsize=7)
    plt.rc('lines', lw=0.2, mew=0.2)
    plt.rc('grid', linewidth=0.2)
    fig = plt.figure(figsize=(10,6))
    plt.subplots_adjust(left=0.07, right=0.95, bottom=0.1, top=0.96, hspace=0.2, wspace=0)
    print('Plot active time for station 1...')
    ax1 = fig.add_subplot(511)
    ax1.bar(dates,st1,width=0.01,edgecolor='none',color='green')
    ax1.tick_params(direction='in')
    ax1.set_ylabel('Station 1')
    ax1.xaxis_date()
    plt.yticks([])
    ax1.grid()
    print('Plot active time for station 2...')
    ax = fig.add_subplot(512,sharex=ax1,sharey=ax1)
    ax.bar(dates,st2,width=0.01,edgecolor='none',color='green')
    ax.tick_params(direction='in')
    ax.set_ylabel('Station 2')
    ax.xaxis_date()
    plt.yticks([])
    ax.grid()
    print('Plot active time for station 3...')
    ax = fig.add_subplot(513,sharex=ax1,sharey=ax1)
    ax.bar(dates,st3,width=0.01,edgecolor='none',color='green')
    ax.tick_params(direction='in')
    ax.set_ylabel('Station 3')
    ax.xaxis_date()
    plt.yticks([])
    ax.grid()
    print('Plot active time for station 4...')
    ax = fig.add_subplot(514,sharex=ax1,sharey=ax1)
    ax.bar(dates,st4,width=0.01,edgecolor='none',color='green')
    ax.tick_params(direction='in')
    ax.set_ylabel('Station 4')
    ax.xaxis_date()
    plt.yticks([])
    ax.grid()
    print('Plot active time for station 0...')
    ax = fig.add_subplot(515,sharex=ax1,sharey=ax1)
    ax.bar(dates,st0,width=0.01,edgecolor='none',color='green')
    ax.tick_params(direction='in')
    ax.set_ylabel('Station 0')
    ax.xaxis_date()
    plt.yticks([])
    ax.grid()
    ax.set_xlabel(r'Hourly activity in %s %i (UTC)'%(d0.strftime("%B"),y0))
    ax1.xaxis.set_major_formatter(md.DateFormatter('%d'))
    ax1.xaxis.set_major_locator(md.DayLocator())
    ax1.set_xlim(d0,d1)
    ax1.set_ylim(0,1)
    plt.savefig('%i-%02i.pdf'%(y0,m0),dpi=80)

if __name__=='__main__':
    from nuri import get_args
    args = get_args()
    active(args.date)



#!/usr/bin/env python
from distutils.core import setup
import os,nuri.__init__
from glob import glob

#del os.link
setup(
    name="nuri",
    version=nuri.__version__,
    author="Vincent Dumont",
    author_email="vincentdumont@gmail.com",
    packages=["nuri"],
    dependency_links=['git+https://gitlab.com/citymag/analysis/mlpy.git'],
    scripts = glob('bin/*'),
    url="http://citymag.gitlab.io/nuri/",
    description="Urban Magnetometry Software"
)



###############################################################################
# Copyright (c) 2018, Urban Magnetometry Group.
#
# Produced at the University of California at Berkeley.
#
# Written by V. Dumont (vincentdumont11@gmail.com).
#
# All rights reserved.
#
# This file is part of NURI.
#
# For details, see github.com/vincentdumont/nuri.
#
# For details about use and distribution, please read NURI/LICENSE.
###############################################################################
def split_data():
    '''
    Take measurement files obtained from sample grabber and split
    them into individual hour files.
    
    Parameters
    ----------
    files : str
      Time data file 
    '''
    os.system('mkdir -p split_data')
    imag  = 0
    itime = 0
    dates = []
    tdata = open('2016-11-10_23-xx_time_v2.bin','rb')
    xdata = open('2016-11-10_23-xx_rawX_uT_3960Hz.bin','rb')
    ydata = open('2016-11-10_23-xx_rawY_uT_3960Hz.bin','rb')
    zdata = open('2016-11-10_23-xx_rawZ_uT_3960Hz.bin','rb')
    while True:
        try:
            tdata.seek(itime)
        except ValueError:
            tchunk.close()
            xchunk.close()
            ychunk.close()
            zchunk.close()
            break
        data = struct.unpack('<'+'qiBQddcdcdd',tdata.read(63))
        npts = int(data[1])
        tbeg = float(data[4])
        tbeg = datetime.fromtimestamp(tbeg).strftime('%Y-%-m-%-d_%-H')
        if tbeg not in dates:
            print(tbeg)
            dates.append(tbeg)
            if len(dates)>1:
                tchunk.close()
                xchunk.close()
                ychunk.close()
                zchunk.close()
            tchunk = open('split_data/%s-xx_time_v2.bin'%tbeg,'wb')
            xchunk = open('split_data/%s-xx_rawX_uT_3960Hz.bin'%tbeg,'wb')
            ychunk = open('split_data/%s-xx_rawY_uT_3960Hz.bin'%tbeg,'wb')
            zchunk = open('split_data/%s-xx_rawZ_uT_3960Hz.bin'%tbeg,'wb')
        data = [0,float(data[1]),float(data[2]),float(data[3]),
                float(data[4]),float(data[5]),str(data[6]),
                float(data[7]),str(data[8]),float(data[9]),
                float(data[10])]
        tchunk.write(struct.pack('<'+'qiBQddcdcdd',*data))
        itime+=63
        for i in range(npts):
            xdata.seek(imag)
            xchunk.write(xdata.read(8))
            ydata.seek(imag)
            ychunk.write(ydata.read(8))
            zdata.seek(imag)
            zchunk.write(zdata.read(8))
            imag+=8
        



###############################################################################
# Copyright (c) 2018, Urban Magnetometry Group.
#
# Produced at the University of California at Berkeley.
#
# Written by V. Dumont (vincentdumont11@gmail.com).
#
# All rights reserved.
#
# This file is part of NURI.
#
# For details, see github.com/vincentdumont/nuri.
#
# For details about use and distribution, please read NURI/LICENSE.
###############################################################################
def timing():
    
    rep = '%s_%s'%(setup.start,setup.end)
    os.system('mkdir -p %s'%rep)
    os.chdir(rep)
    #----------------------------------------------------------
    # Extract magnetic field data
    #----------------------------------------------------------
    # Get NURI data
    path = '/Users/vincent/ASTRO/data/NURI/timing/run%i/station2/'%setup.run
    nuri = magfield(setup.start,setup.end,rep=path)
    nuri_pulse = (max(nuri[:,1])+min(nuri[:,1]))/2.
    # Get GNOME data
    path = '/Users/vincent/ASTRO/data/GNOME/timing/'
    gnome = gnomedata('Berkeley02',setup.start,setup.end,rep=path)
    gnome_pulse = (max(gnome[:,1])+min(gnome[:,1]))/2.
    # Get start and ending timestamps
    if setup.sample==False or setup.rescale:
        tmin = max(nuri[0,0],gnome[0,0])
        tmax = min(nuri[-1,0],gnome[-1,0])
    else:
        start,tmin,end,tmax = time_edge(setup.start,setup.end)
    #----------------------------------------------------------
    # Identify timestamps of every pulse for every station
    #----------------------------------------------------------
    for n in range(2):
        if n==0: magdata,pulse = nuri,nuri_pulse
        if n==1: magdata,pulse = gnome,gnome_pulse
        flag,data = 0,[]
        for i in range(1,len(magdata)-1):
            pulse_up   = magdata[i-1,1] < pulse <= magdata[i,1]
            pulse_down = magdata[i+1,1] < pulse <= magdata[i,1]
            if tmin < magdata[i,0] and flag==0 and pulse_up:
                data.append(magdata[i,0])
                flag = 1
            if tmin < magdata[i,0] and flag==1 and pulse_down:
                flag = 0
        if n==0: pulses_nuri  = data
        if n==1: pulses_gnome = data
    #----------------------------------------------------------
    # Calculate pulse delay between for NURI and GNOME data
    #----------------------------------------------------------
    diff_nuri = diff_gnome = delay = np.empty((0,2))
    imax = min(len(pulses_nuri),len(pulses_gnome))
    for i in range(1,imax):
        x = pulses_nuri[i]-tmin
        y = pulses_nuri[i]-pulses_nuri[i-1]
        diff_nuri = np.vstack((diff_nuri,[x,y]))
        x = pulses_gnome[i]-tmin
        y = pulses_gnome[i]-pulses_gnome[i-1]
        diff_gnome = np.vstack((diff_gnome,[x,y]))
        x = pulses_nuri[i]-tmin
        y = pulses_nuri[i]-pulses_gnome[i]
        delay = np.vstack((delay,[x,y]))
    #----------------------------------------------------------
    # Create x axis time label
    #----------------------------------------------------------
    tbeg = datetime.fromtimestamp(tmin)
    tend = datetime.fromtimestamp(tmax)
    sbeg = tbeg.strftime('%Y-%m-%d %H:%M:%S.%f')
    send = tend.strftime('%Y-%m-%d %H:%M:%S.%f')
    label1 = 'UTC time from %s to %s [secs]'%(sbeg,send)
    label2 = '%s-%s.png'%(int(tmin),int(tmax))
    #----------------------------------------------------------
    # Histogram of pulse separation
    #----------------------------------------------------------
    diffs  = [(diff_nuri[:,1]-1)*1e6,(diff_gnome[:,1]-1)*1e6]
    labels = ['NURI','GNOME']
    xmin   = min([min(diffs[i]) for i in range(2)])
    xmax   = max([max(diffs[i]) for i in range(2)])
    offset = (xmax-xmin)/10
    xmin   = xmin-offset
    xmax   = xmax+offset
    fig = figure(figsize=(12,6))
    fig.suptitle('Time separation between pulses\n%s'%label1,
                 fontsize=14, fontweight='bold')
    plt.subplots_adjust(left=0.1, right=0.95,
                        bottom=0.1, top=0.85,
                        hspace=0.1, wspace=0)
    ax = plt.subplot(111,xlim=[xmin,xmax])
    ax.hist(diffs,bins=50,fill=True,alpha=0.4,
            edgecolor="black",range=[xmin,xmax],align='left',label=labels)
    ax.legend(loc='center',bbox_to_anchor=[0.5,1.04],
              ncol=2)
    print(np.average(diffs[0]))
    print(np.average(diffs[1]))
    ax.set_xlabel('Pulse delay relative to 1 second [us]')
    ax.set_yscale('log')
    ax.set_ylim(ymin=0.1)
    savefig('histogram_pulses.png')
    clf()
    #----------------------------------------------------------
    # Plot pulses separations and delay
    #----------------------------------------------------------
    fig = figure(figsize=(15,10))
    plt.subplots_adjust(left=0.1,right=0.95,
                        bottom=0.05,top=0.97,
                        hspace=0.1,wspace=0)
    # Plot NURI pulse difference
    ymin,ymax = min(diff_nuri[:,1]),max(diff_nuri[:,1])
    offset    = (ymax-ymin)/10
    ymin,ymax = ymin-offset,ymax+offset
    ax1 = subplot(311,xlim=[0,tmax-tmin],ylim=[ymin,ymax])
    ax1.scatter(diff_nuri[:,0],diff_nuri[:,1],lw=0,s=20,c='#4e73ae')
    ax1.set_ylabel('NURI Pulse separation [secs]')
    plt.setp(ax1.get_xticklabels(),visible=False)
    # Plot GNOME pulse difference
    ymin,ymax = min(diff_gnome[:,1]),max(diff_gnome[:,1])
    offset    = (ymax-ymin)/10
    ymin,ymax = ymin-offset,ymax+offset
    ax2 = subplot(312,sharex=ax1,sharey=ax1)
    ax2.scatter(diff_gnome[:,0],diff_gnome[:,1],lw=0,s=20,c='#4e73ae')
    ax2.set_ylabel('GNOME Pulse separation [secs]')
    ax2.set_ylim([ymin,ymax])
    plt.setp(ax2.get_xticklabels(),visible=False)
    # Plot NURI/GNOME pulse delay
    ymin,ymax = min(delay[:,1]),max(delay[:,1])
    offset    = (ymax-ymin)/10
    ymin,ymax = ymin-offset,ymax+offset
    ax3 = subplot(313,sharex=ax1)
    ax3.scatter(delay[:,0],delay[:,1],lw=0,s=20,c='#4e73ae')
    ax3.set_ylabel('Pulse delay [secs]')
    ax3.set_xlim([0,tmax-tmin]) 
    ax3.set_ylim([ymin,ymax])
    ax3.set_xlabel(label1)
    # Save figure
    plt.savefig('pulses.png')
    plt.close()
    #----------------------------------------------------------
    # Create time series movie
    #----------------------------------------------------------
    os.system('mkdir -p video')
    tstep = 1. #2*tdiff
    t0,t1 = tmin-setup.tdiff,tmin+setup.tdiff
    while t1<min(nuri[-1,0],gnome[-1,0]):
        tmid = (t0+t1)/2
        fig = figure(figsize=(15,7))
        plt.subplots_adjust(left=0.08,right=0.95,
                            bottom=0.1,top=0.97,
                            hspace=0.1,wspace=0)
        # Identify first and last indexes in NURI data
        imin = abs(nuri[:,0]-t0).argmin()-2
        imax = abs(nuri[:,0]-t1).argmin()+2
        # Plot NURI time series
        ax1 = subplot(211)
        ax1.plot((nuri[imin:imax,0]-tmid)*1000.,nuri[imin:imax,1])
        ax1.set_ylim(min(nuri[:,1]),max(nuri[:,1]))
        ax1.set_ylabel('NURI Magnetic Field [uT]')
        plt.setp(ax1.get_xticklabels(),visible=False)
        # Identify first and last indexes in GNOME data
        imin = abs(gnome[:,0]-t0).argmin()-2
        imax = abs(gnome[:,0]-t1).argmin()+2
        # Plot GNOME time series
        ax2 = subplot(212,sharex=ax1)
        ax2.plot((gnome[imin:imax,0]-tmid)*1000.,gnome[imin:imax,1])
        ax2.set_ylim(min(gnome[:,1]),max(gnome[:,1]))
        ax2.set_ylabel('GNOME Magnetic Field [uT]')
        # Set x axis limit and label
        ax2.set_xlim((t0-tmid)*1000.,(t1-tmid)*1000.) 
        date = datetime.fromtimestamp(tmid).strftime('%Y-%m-%d %H:%M:%S.%f')
        ax2.set_xlabel('Time series around %s UTC time [ms]'%(date))
        # Save and close figure
        plt.savefig('video/%s-%s.png'%(t0,t1))
        plt.close()
        t0 += tstep
        t1 += tstep
